/***************************************************************************
 *   Copyright (C) 2008 by S. MANKOWSKI / G. DE BURE skrooge@mankowski.fr  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>  *
 ***************************************************************************/
/** @file
 * This file implements classes SKGDocument.
 *
 * @author Stephane MANKOWSKI / Guillaume DE BURE
 */
#include "skgdocument.h"
#include "skgtraces.h"
#include "skgerror.h"
#include "skgservices.h"
#include "skgobjectbase.h"
#include "skgnamedobject.h"

#include <klocale.h>

#include <QSqlDatabase>
#include <QSqlQuery>
#include <QSqlError>
#include <QSqlDriver>
#include <QHash>
#include <QRegExp>
#include <QUuid>
#include <QVariant>
#include <QFile>
#include <QDir>

#include <sqlite3.h>

SKGError SKGDocument::lastCallbackError;
int SKGDocument::databaseUniqueIdentifier=0;

SKGDocument::SKGDocument(DatabaseMode iMode)
                :QObject(), lastSavedTransaction(0), progressFunction(NULL), progressData(NULL),
                currentFileName(""), currentDatabase(NULL),  inundoRedoTransaction(0), currentTransaction(0),
                mode(iMode),inProgress(false)
{
        SKGTRACEIN(10, "SKGDocument::SKGDocument");

        //Initialisation of undoable tables
        SKGListNotUndoable.push_back("T.doctransaction");
        SKGListNotUndoable.push_back("T.doctransactionitem");
        SKGListNotUndoable.push_back("T.doctransactionmsg");

        //Database unique identifier
        ++databaseUniqueIdentifier;
        databaseIdentifier="SKGDATABASE_"+SKGServices::intToString(databaseUniqueIdentifier);

        //Initialisation of backup file parameters
        setBackupParameters("", ".old");
}


SKGDocument::~SKGDocument()
{
        SKGTRACEIN(10, "SKGDocument::~SKGDocument");
        close();
        progressFunction=NULL;
        progressData=NULL;
}

QString SKGDocument::getUniqueIdentifier()
{
        return uniqueIdentifier;
}

SKGDocument::DatabaseMode SKGDocument::getDatabaseMode()
{
        return mode;
}

SKGError SKGDocument::setProgressCallback(int (*iProgressFunction)(int, void*), void* iData)
{
        SKGError err;
        progressFunction=(void*) iProgressFunction;
        progressData=iData;

        return err;
}

SKGError SKGDocument::stepForward(int iPosition)
{
        SKGError err;

        //Increase the step for the last transaction
        if (getDepthTransaction()) {
                posStepForTransaction.pop_back();
                posStepForTransaction.push_back(iPosition);
        }

        //Check if a callback function exists
        if (progressFunction) {
                //YES ==> compute
                double min=0;
                double max=100;

                bool emitevent=true;
                SKGIntListIterator nbIt=nbStepForTransaction.begin();
                SKGIntListIterator posIt=posStepForTransaction.begin();
                for (; emitevent && nbIt!=nbStepForTransaction.end(); ++nbIt) {
                        int p=*posIt;
                        int n=*nbIt;
                        if (p<0 || p>n) p=n;

                        if (n!=0) {
                                double pmin=min;
                                double pmax=max;
                                min=pmin+(pmax-pmin)*(static_cast<double>(p) / static_cast<double>(n));
                                max=pmin+(pmax-pmin)*(static_cast<double>(p+1) / static_cast<double>(n));
                                if (max>100) max=100;
                        } else emitevent=false;

                        ++posIt;
                }

                int posPourcent=(int) min;

                //Call the call back
                if (emitevent) {
                        inProgress=true;
                        if ((*(int (*)(int, void*)) progressFunction)(posPourcent, progressData)!=0) {
                                err.setReturnCode(ERR_ABORT);
                                err.setMessage(i18n("The current operation has been interrupted"));

                                //Remove all untransactionnal message
                                unTransactionnalMessages.clear();
                        }
                        inProgress=false;
                }
        }
        return err;
}

SKGError SKGDocument::beginTransaction(const QString & iName, int iNbStep, const QDateTime& iDate)
{
        SKGError err;
        SKGTRACEINRC(5, "SKGDocument::beginTransaction", err);
        SKGTRACEL(10) << "Input parameter [name]=[" << iName << ']' << endl;
        if (nbStepForTransaction.size() == 0) {
                //Open SQLtransaction
                err = SKGServices::executeSqliteOrder(this, "BEGIN;");
                if (err.isSucceeded()) {
                        //Create undo redo transaction
                        err = SKGServices::executeSqliteOrder(this, "insert into doctransaction "
                                                              "(d_date, t_name, i_parent) values "
                                                              "('"+SKGServices::timeToString(iDate)+
                                                              "','"+SKGServices::stringToSqlString(iName)+
                                                              "', "+SKGServices::intToString(getTransactionToProcess(SKGDocument::UNDO))+");");
                        currentTransaction=getTransactionToProcess(SKGDocument::UNDO);
                }
        } else {
                //A transaction already exists
                //Check if the child transaction is a opened in the progress callback
                if (inProgress) {
                        err.setReturnCode(ERR_FAIL);
                        err.setMessage(i18n("A transaction can not be started during execution of an other one"));
                }
        }
        if (err.isSucceeded()) {
                nbStepForTransaction.push_back(iNbStep);
                posStepForTransaction.push_back(iNbStep);

                if (iNbStep) err=stepForward(0);
        }
        return err;
}

SKGError SKGDocument::checkExistingTransaction() const
{
        SKGError err;
        SKGTRACEINRC(10, "SKGDocument::checkExistingTransaction", err);
        if (getDepthTransaction() <= 0) {
                err.setReturnCode(ERR_ABORT);
                err.setMessage(i18n("A transaction must be opened to do this action"));
        }
        return err;
}

SKGError SKGDocument::endTransaction(bool succeedded)
{
        SKGError err;
        SKGTRACEINRC(5, "SKGDocument::endTransaction", err);
        if (nbStepForTransaction.size()==0) {
                //Set error message
                err.setReturnCode(ERR_ABORT);
                err.setMessage(i18n("Closing transaction failed because too many transactions ended"));
        } else {
                stepForward(nbStepForTransaction.at(nbStepForTransaction.count()-1)); //=100%
                QStringList listModifiedTables;
                nbStepForTransaction.pop_back();
                posStepForTransaction.pop_back();
                QString currentTransactionString=SKGServices::intToString(getCurrentTransaction());

                if (nbStepForTransaction.size()==0) {
                        if (succeedded) {
                                //Link items on current transaction
                                if (err.isSucceeded()) {
                                        err = SKGServices::executeSqliteOrder(this, "UPDATE doctransactionitem set rd_doctransaction_id="+currentTransactionString+" WHERE rd_doctransaction_id=0;");
                                }

                                //Optimization of the current transaction
                                if (err.isSucceeded()) {
                                        SKGStringListList listTmp;
                                        err = SKGServices::executeSelectSqliteOrder(this, "SELECT count(1) FROM doctransactionitem where rd_doctransaction_id="+currentTransactionString, listTmp);
                                        if (err.isSucceeded()) {
                                                int nbItem=SKGServices::stringToInt(listTmp.at(1).at(0));
                                                if (nbItem==0) {
                                                        //Optimization is needed
                                                        //Get popup messages
                                                        QStringList popupMessages;
                                                        getMessages(getCurrentTransaction(), popupMessages, false);

                                                        //Delete current transaction
                                                        err = SKGServices::executeSqliteOrder(this, "DELETE FROM doctransaction WHERE id="+currentTransactionString);

                                                        foreach(const QString& msg, popupMessages) {
                                                                unTransactionnalMessages.push_back(msg);
                                                        }
                                                }
                                        }

                                }

                                //Optimization 2: remove duplicate orders
                                if (err.isSucceeded()) {
                                        err = SKGServices::executeSqliteOrder(this,
                                                                              "DELETE FROM doctransactionitem WHERE id IN "
                                                                              "(SELECT a.id FROM doctransactionitem a, doctransactionitem b "
                                                                              "WHERE a.rd_doctransaction_id="+currentTransactionString+
                                                                              " AND b.rd_doctransaction_id=a.rd_doctransaction_id AND a.i_object_id=b.i_object_id "
                                                                              "AND a.t_object_table=b.t_object_table AND "
                                                                              "b.t_action=a.t_action AND b.t_sqlorder=a.t_sqlorder AND a.id>b.id );");
                                }

                                //Get current transaction information to be able to emit envent in case of SKG_UNDO_MAX_DEPTH=0
                                if (err.isSucceeded()) {
                                        err=SKGServices::getDistinctValues(this, "doctransactionitem",
                                                                           "t_object_table",
                                                                           "rd_doctransaction_id="+currentTransactionString,
                                                                           listModifiedTables);
                                }

                                //Remove oldest transaction
                                if (err.isSucceeded()) {
                                        QString maxdepthstring = getParameter("SKG_UNDO_MAX_DEPTH");
                                        if (maxdepthstring.length() == 0) maxdepthstring = "-1";
                                        int maxdepth = SKGServices::stringToInt(maxdepthstring);
                                        if (maxdepth >= 0) {
                                                err = SKGServices::executeSqliteOrder(this, "delete from doctransaction where id in (select id from doctransaction limit max(0,((select count(1) from doctransaction)-(" + maxdepthstring + "))))");
                                        }
                                }

                                //Remove SKGDocument::REDO transactions if we are not in a undo / redo transaction
                                if (!inundoRedoTransaction) {
                                        int i = 0;
                                        while ((i = getTransactionToProcess(SKGDocument::REDO)) && err.isSucceeded()) {
                                                err = SKGServices::executeSqliteOrder(this, "delete from doctransaction where id=" + SKGServices::intToString(i));
                                        }
                                }

                                //Commit the transaction
                                if (err.isSucceeded()) {
                                        err = SKGServices::executeSqliteOrder(this, "COMMIT;");
                                }

                        }

                        if (!succeedded || err.isFailed()) {
                                //Rollback the transaction
                                SKGError err2 = SKGServices::executeSqliteOrder(this, "ROLLBACK;");
                                if (err2.isSucceeded()) {
                                        //delete the transaction
                                        err2 = SKGServices::executeSqliteOrder(this, "delete from doctransaction where id=" + currentTransactionString);
                                }

                                if (err2.isFailed()) err.addError(err2.getReturnCode(), err2.getMessage());
                        } else {
                                //For better performance, events are submitted only for the first recursive undo
                                if (inundoRedoTransaction<=1) {
                                        //Emit modification events
                                        QStringList tablesRefreshed;
                                        foreach (const QString& table, listModifiedTables) {
                                                emit tableModified(table, getCurrentTransaction());
                                                tablesRefreshed.push_back(table);
                                        }
                                        emit tableModified("doctransaction", getCurrentTransaction());
                                        emit tableModified("doctransactionitem", getCurrentTransaction());

                                        //WARNING: list is modified during treatement
                                        for (int i=0; i<listModifiedTables.count(); ++i) {
                                                QString table=listModifiedTables[i];
                                                if (!tablesRefreshed.contains(table)) {
                                                        emit tableModified(table, 0);
                                                        tablesRefreshed.push_back(table);
                                                }

                                                QStringList toAdd=getImpactedTable(table);
                                                int nbToAdd=toAdd.count();
                                                for (int j=0; j<nbToAdd; ++j) {
                                                        if (!listModifiedTables.contains(toAdd.at(j)))
                                                                listModifiedTables.push_back(toAdd.at(j));
                                                }
                                        }

                                        emit transactionSuccessfullyEnded(getCurrentTransaction());

                                        //Remove temporary transaction if needed
                                        err = SKGServices::executeSqliteOrder(this, "delete from doctransaction where id="+currentTransactionString+" and t_name='#INTERNAL#';");
                                }
                        }

                        currentTransaction=0;

                        //clean cache
                        cache.clear();
                }
        }
        return err;
}

SKGError SKGDocument::sendMessage(const QString& iMessage, bool iPopup)
{
        SKGError err;
        SKGTRACEINRC(10, "SKGDocument::sendMessage", err);
        //Associate message with transaction
        if (checkExistingTransaction().isSucceeded()) {
                SKGObjectBase msg(this, "doctransactionmsg");
                err = msg.setAttribute("rd_doctransaction_id", SKGServices::intToString(getCurrentTransaction()));
                if (err.isSucceeded()) err = msg.setAttribute("t_message", iMessage);
                if (err.isSucceeded()) err = msg.setAttribute("t_popup", iPopup ? "Y" : "N");
                if (err.isSucceeded()) err = msg.save();
        } else {
                //Addition message in global variable in case of no transaction opened
                if (iPopup) unTransactionnalMessages.push_back(iMessage);
        }
        return err;
}

SKGError SKGDocument::getMessages(int iIdTransaction, QStringList& oMessages, bool iAll)
{
        SKGError err;
        SKGTRACEINRC(10, "SKGDocument::getMessages", err);
        oMessages=unTransactionnalMessages;
        unTransactionnalMessages.clear();

        SKGStringListList listTmp;
        err=SKGServices::executeSelectSqliteOrder(this,
                        QString("SELECT t_message, t_popup FROM doctransactionmsg WHERE ")+
                        (iAll ? "t_popup IS NOT NULL" : "t_popup='Y'")+
                        " AND rd_doctransaction_id=" +
                        SKGServices::intToString(iIdTransaction) +
                        " ORDER BY id ASC",
                        listTmp);
        int nb=listTmp.count();
        for (int i=1; err.isSucceeded() && i<nb ;++i) {
                QString msg=listTmp.at(i).at(0);
                if (!oMessages.contains(msg)) oMessages.push_back(msg);
        }
        return err;
}

SKGError SKGDocument::getModifications(int iIdTransaction, SKGObjectModificationList& oModifications) const
{
        SKGError err;
        SKGTRACEINRC(10, "SKGDocument::getModifications", err);
        oModifications.clear();

        SKGStringListList listTmp;
        err=SKGServices::executeSelectSqliteOrder(this,
                        "SELECT i_object_id,t_object_table,t_action FROM doctransactionitem WHERE rd_doctransaction_id=" +
                        SKGServices::intToString(iIdTransaction) +
                        " ORDER BY id ASC",
                        listTmp);
        int nb=listTmp.count();
        for (int i=1; err.isSucceeded() && i<nb ;++i) {
                SKGObjectModification mod;
                mod.id=SKGServices::stringToInt(listTmp.at(i).at(0));
                mod.table=listTmp.at(i).at(1);
                QString type=listTmp.at(i).at(2);
                mod.type=(type=="D" ? I : (type=="I" ? D : U )); //Normal because in database we have to sql order to go back.
                mod.uuid=listTmp.at(i).at(0)+ '-' + mod.table;;

                oModifications.push_back(mod);
        }
        return err;
}

QStringList SKGDocument::getImpactedTable(const QString& /*iTable*/) const
{
        SKGTRACEIN(10, "SKGDocument::getImpactedTable");
        QStringList output;
        return output;
}

SKGError SKGDocument::groupTransactions(int iFrom, int iTo)
{
        SKGError err;
        SKGTRACEINRC(5, "SKGDocument::groupTransactions", err);

        ++inundoRedoTransaction; //It's a kind of undo redo

        //Check if a transaction is still opened
        err = checkExistingTransaction();
        if (err.isSucceeded()) {
                //A transaction is still opened
                err.setReturnCode(ERR_ABORT);
                err.setMessage(i18n("Creation of a group of transactions is forbidden inside a transaction"));

        } else {
                int iidMaster=qMax(iFrom, iTo);
                QString smin=SKGServices::intToString(qMin(iFrom, iTo));
                QString smax=SKGServices::intToString(iidMaster);

                //Get transaction
                SKGStringListList transactions;
                err=SKGServices::executeSelectSqliteOrder(this,
                                QString("SELECT id, t_name, t_mode, i_parent FROM doctransaction WHERE id BETWEEN ") +
                                smin +" AND " +
                                smax+ " ORDER BY id ASC",
                                transactions);

                //Check and get main parameter for the group
                int nb=transactions.count();
                QString transactionMode;
                QString communParent;
                QString name;
                for (int i=1; err.isSucceeded() && i<nb; ++i) { //We forget header
                        QStringList transaction=transactions.at(i);
                        QString mode=transaction.at(2);
                        if (!name.isEmpty()) name+=',';
                        name+=transaction.at(1);

                        if (!transactionMode.isEmpty() && mode!=transactionMode)  err=SKGError(ERR_INVALIDARG, "Undo and Redo transactions can not be grouped");
                        else  transactionMode=mode;

                        if (i==1)  communParent=transaction.at(3);
                }


                if (err.isSucceeded()) {
                        //Group
                        err = SKGDocument::beginTransaction("#INTERNAL#");
                        if (err.isSucceeded()) {
                                //Group items
                                if (err.isSucceeded())
                                        err = SKGServices::executeSqliteOrder(this,
                                                                              QString("UPDATE doctransactionitem set rd_doctransaction_id=") +
                                                                              smax +
                                                                              " where rd_doctransaction_id BETWEEN " +
                                                                              smin+ " AND " + smax);
                                if (err.isSucceeded())
                                        err = SKGServices::executeSqliteOrder(this,
                                                                              QString("UPDATE doctransaction set i_parent=") +
                                                                              communParent +
                                                                              ", t_name='"+SKGServices::stringToSqlString(name)+
                                                                              "' where id=" +smax);

                                if (err.isSucceeded()) err = SKGServices::executeSqliteOrder(this,
                                                                     QString("DELETE FROM doctransaction WHERE id BETWEEN ") +
                                                                     smin+ " AND " + SKGServices::intToString(qMax(iFrom, iTo)-1));
                        }

                        if (err.isSucceeded()) err=endTransaction(true);
                        else  endTransaction(false);
                }
        }

        --inundoRedoTransaction;
        return err;
}

SKGError SKGDocument::undoRedoTransaction(const UndoRedoMode& iMode)
{
        SKGError err;
        SKGTRACEINRC(5, "SKGDocument::undoRedoTransaction", err);
        //Check if a transaction is still opened
        err = checkExistingTransaction();
        if (err.isSucceeded()) {
                //A transaction is still opened
                err.setReturnCode(ERR_ABORT);
                err.setMessage(i18n("Undo / Redo is forbidden inside a transaction"));

        } else {
                if (iMode==SKGDocument::UNDOLASTSAVE) {
                        //Create group
                        SKGStringListList transactions;
                        err=SKGServices::executeSelectSqliteOrder(this,
                                        "SELECT id, t_savestep FROM doctransaction WHERE t_mode='U' ORDER BY id DESC",
                                        transactions);
                        int nb=transactions.count();
                        int min=0;
                        int max=0;
                        for (int i=1; err.isSucceeded() && i<nb; ++i) {
                                QStringList transaction=transactions.at(i);
                                if (i==1) {
                                        max=SKGServices::stringToInt(transaction.at(0));
                                }
                                if (i!=1 && transaction.at(1)=="Y") {
                                        break;
                                }
                                min=SKGServices::stringToInt(transaction.at(0));
                        }
                        if (min==0) min=max;
                        if (err.isSucceeded() && min!=max && min!=0) err=groupTransactions(min, max);
                } else {
                        err=SKGError(); //To ignore error generated by checkExistingTransaction.
                }

                //Get ID of the transaction to undo
                if (err.isSucceeded()) {
                        QString name;
                        bool saveStep=false;
                        QDateTime date;
                        int id = getTransactionToProcess(iMode, &name, &saveStep, &date);
                        if (id == 0) {
                                //No transaction found ==> generate an error
                                err = SKGError(ERR_INVALIDARG, "No transaction found. Undo / Redo impossible.");
                        } else {

                                //Undo transaction
                                SKGTRACEL(5) << "Undoing transaction [" << id << "]- [" << name << "]..." << endl;
                                SKGStringListList listSqlOrder;
                                err=SKGServices::executeSelectSqliteOrder(this,
                                                "SELECT t_sqlorder FROM doctransactionitem WHERE rd_doctransaction_id=" +
                                                SKGServices::intToString(id) +
                                                " ORDER BY id DESC",
                                                listSqlOrder);
                                if (err.isSucceeded()) {
                                        int nb=listSqlOrder.count();
                                        err = SKGDocument::beginTransaction(name, nb+3, date);
                                        if (err.isSucceeded()) {
                                                ++inundoRedoTransaction; //Because we will be in a undo/redo transaction
                                                //Normal the first element is ignored because it is the header
                                                for (int i=1; err.isSucceeded() && i<nb ;++i) {
                                                        err=SKGServices::executeSqliteOrder(this, listSqlOrder.at(i).at(0));

                                                        if (err.isSucceeded()) err=stepForward(i); //Undo / redo are not cancelable
                                                }

                                                if (err.isSucceeded()) {
                                                        //Set the NEW transaction in redo mode
                                                        int lastredo = getTransactionToProcess((iMode == SKGDocument::UNDO || iMode == SKGDocument::UNDOLASTSAVE  ? SKGDocument::REDO : SKGDocument::UNDO));
                                                        int newredo = getTransactionToProcess(iMode);
                                                        if (err.isSucceeded())
                                                                err = SKGServices::executeSqliteOrder(this,
                                                                                                      QString("UPDATE doctransaction set t_mode=") +
                                                                                                      (iMode == SKGDocument::UNDO || iMode == SKGDocument::UNDOLASTSAVE ? "'R'" : "'U'") +
                                                                                                      ", i_parent=" +
                                                                                                      SKGServices::intToString(lastredo) +
                                                                                                      " where id=" + SKGServices::intToString(newredo));
                                                        if (err.isSucceeded()) err=stepForward(nb);

                                                        //Move messages from previous transaction to new one
                                                        if (err.isSucceeded())
                                                                err = SKGServices::executeSqliteOrder(this,
                                                                                                      "UPDATE doctransactionmsg set rd_doctransaction_id="+
                                                                                                      SKGServices::intToString(getCurrentTransaction()) +
                                                                                                      " where rd_doctransaction_id=" +
                                                                                                      SKGServices::intToString(id));
                                                        if (err.isSucceeded()) err=stepForward(nb+1);

                                                        //delete treated transaction
                                                        if (err.isSucceeded())
                                                                err = SKGServices::executeSqliteOrder(this,
                                                                                                      "DELETE from doctransaction where id="
                                                                                                      + SKGServices::intToString(id));
                                                        if (err.isSucceeded()) err=stepForward(nb+2);

                                                        //Check that new transaction has exactly the same number of item
                                                        /* if (err.isSucceeded()) {
                                                                 SKGStringListList listSqlOrder;
                                                                 err=SKGServices::executeSelectSqliteOrder(this,
                                                                                 "SELECT count(1) FROM doctransactionitem WHERE rd_doctransaction_id=" +
                                                                                 SKGServices::intToString(getCurrentTransaction()),
                                                                                 listSqlOrder);
                                                                 if (err.isSucceeded() && SKGServices::stringToInt(listSqlOrder.at(1).at(0))!=nb-1) {
                                                                         err=SKGError(ERR_ABORT, i18n("Invalid number of item after undo/redo. Expected (%1) != Result (%2)",nb-1,listSqlOrder.at(1).at(0)));
                                                                 }
                                                         }*/

                                                        if (err.isSucceeded()) err=stepForward(nb+3);
                                                }

                                                if (err.isSucceeded()) err=endTransaction(true);
                                                else  endTransaction(false);
                                                --inundoRedoTransaction; //We left the undo / redo transaction
                                        }
                                }
                        }
                }
        }

        return err;
}

int SKGDocument::getDepthTransaction() const
{
        return nbStepForTransaction.size();
}

int SKGDocument::getNbTransaction(const UndoRedoMode& iMode) const
{
        SKGTRACEIN(10, "SKGDocument::getNbTransaction");
        int output = 0;
        if (getDatabase()) {
                QString sqlorder = "select count(1) from doctransaction where t_mode='";
                sqlorder+=(iMode == SKGDocument::UNDO || iMode == SKGDocument::UNDOLASTSAVE ? "U" : "R");
                sqlorder+='\'';
                QSqlQuery query=getDatabase()->exec(sqlorder);
                if (query.next()) {
                        output=query.value(0).toInt();
                }
        }
        return output;
}

int SKGDocument::getTransactionToProcess(const UndoRedoMode& iMode, QString* oName, bool* oSaveStep, QDateTime* oDate) const
{
        SKGTRACEIN(10, "SKGDocument::getTransactionToProcess");
        //initialisation
        int output = 0;
        if (oName) *oName = "";
        if (getDatabase()) {
                QString sqlorder = "select A.id , A.t_name, A.t_savestep, A.d_date from doctransaction A where "
                                   "((select count(1) from doctransaction B where B.i_parent=A.id)=0) "
                                   "and A.t_mode='";
                sqlorder+=(iMode == SKGDocument::UNDO || iMode == SKGDocument::UNDOLASTSAVE ? "U" : "R");
                sqlorder+='\'';
                QSqlQuery query=getDatabase()->exec(sqlorder);
                if (query.next()) {
                        output=query.value(0).toInt();
                        if (oName != NULL) *oName = query.value(1).toString();
                        if (oSaveStep != NULL) *oSaveStep = (query.value(2).toString()=="Y");
                        if (oDate != NULL) *oDate = SKGServices::stringToTime(query.value(3).toString());
                }
        }
        return output;
}

int SKGDocument::getCurrentTransaction() const
{
        SKGTRACEIN(10, "SKGDocument::getCurrentTransaction");
        return currentTransaction;
}

SKGError SKGDocument::changePassword(const QString & newPassword)
{
        SKGError err;
        SKGTRACEINRC(10, "SKGDocument::changePassword", err);
        err = setParameter("SKG_PASSWORD", newPassword);
        if (err.isSucceeded()) {
                if (newPassword.isEmpty()) {
                        err=sendMessage(i18n("The document password has been removed."));
                } else {
                        err=sendMessage(i18n("The document password has been modified."));
                }
        }
        return err;
}

SKGError SKGDocument::setLanguage(const QString& iLanguage)
{
        SKGError err;
        SKGTRACEINRC(5, "SKGDocument::setLanguage", err);
        QString previousLanguage=getParameter("SKG_LANGUAGE");
        if (previousLanguage!=iLanguage) {
                //Save language into the document
                if (err.isSucceeded()) err=beginTransaction("#INTERNAL#");
                if (err.isSucceeded()) err=setParameter("SKG_LANGUAGE", iLanguage);

                //Migrate view for new language
                if (err.isSucceeded()) err=refreshViewsIndexesAndTriggers();

                //close temporary transaction
                if (err.isSucceeded()) err=endTransaction(true);
                else  endTransaction(false);
        }
        return err;
}

SKGError SKGDocument::initialize()
{
        SKGError err;
        SKGTRACEINRC(5, "SKGDocument::initialize", err);
        err=load("", "");
        return err;
}

SKGError SKGDocument::load(const QString & name, const QString & password, bool restoreTmpFile)
{
        //close previous document
        SKGError err;
        SKGTRACEINRC(5, "SKGDocument::load", err);
        SKGTRACEL(10) << "Input parameter [name]=[" << name << ']' << endl;

        DatabaseMode dbMode=getDatabaseMode();
        lastSavedTransaction=-1; //To avoid double event emission
        err=close();
        if (err.isSucceeded()) {
                //Check if file exists
                QFile inputFile(name);
                if (name.length() && !inputFile.exists()) {
                        //Set error message
                        err.setReturnCode(ERR_INVALIDARG);
                        err.setMessage(i18n("File [%1] not found",name));
                } else {
                        // open your memory database
                        if (dbMode==CopiedInMemory) {
                                //Create memory database
                                currentDatabase=new QSqlDatabase(QSqlDatabase::addDatabase("QSQLITE", databaseIdentifier));
                                currentDatabase->setDatabaseName(":memory:");
                                if (!currentDatabase->open()) {
                                        //Set error message
                                        QSqlError sqlErr=currentDatabase->lastError();
                                        err = SKGError(SQLLITEERROR + sqlErr.number(), sqlErr.text());

                                        //close database
                                        close();
                                }
                        }

                        if (err.isSucceeded()) {
                                if (inputFile.exists()) {
                                        //File exist
                                        if (dbMode==DirectAccess) {
                                                //Direct load of existing file
                                                currentDatabase=new QSqlDatabase(QSqlDatabase::addDatabase("QSQLITE", databaseIdentifier));
                                                currentDatabase->setDatabaseName(name);
                                                if (!currentDatabase->open()) {
                                                        //Set error message
                                                        QSqlError sqlErr=currentDatabase->lastError();
                                                        err = SKGError(SQLLITEERROR + sqlErr.number(), sqlErr.text());
                                                }
                                        } else if (dbMode==CopiedInFile) {
                                                //Copy in temporary file

                                                temporaryFile=SKGDocument::getTemporaryFile(name);
                                                if (!restoreTmpFile) {
                                                        QFile(temporaryFile).remove(); //Must must remove it to be able to copy
                                                        err=SKGServices::cryptFile(name, temporaryFile, password, false);
                                                }
                                                if (err.isSucceeded()) {
                                                        currentDatabase=new QSqlDatabase(QSqlDatabase::addDatabase("QSQLITE", databaseIdentifier));
                                                        currentDatabase->setDatabaseName(temporaryFile);
                                                        if (!currentDatabase->open()) {
                                                                //Set error message
                                                                QSqlError sqlErr=currentDatabase->lastError();
                                                                err = SKGError(SQLLITEERROR + sqlErr.number(), sqlErr.text());
                                                        }
                                                }
                                        } else {
                                                //Copy in memory
                                                {//This scope is needed to avoid a warning at remove
                                                        QSqlDatabase dbfile = QSqlDatabase::addDatabase("QSQLITE", "TEMPORARY_LOAD");
                                                        dbfile.setDatabaseName(name);
                                                        if (!dbfile.open()) {
                                                                //Set error message
                                                                QSqlError sqlErr=dbfile.lastError();
                                                                err = SKGError(SQLLITEERROR + sqlErr.number(), sqlErr.text());
                                                        } else {
                                                                //Copy db from file to memory
                                                                err = SKGServices::copySqliteDatabase(&dbfile, getDatabase(), true);

                                                                //Close
                                                                dbfile.close();
                                                        }
                                                }
                                                //Remove database
                                                QSqlDatabase::removeDatabase("TEMPORARY_LOAD");
                                        }

                                        currentFileName=name;
                                } else {
                                        if (dbMode==DirectAccess || dbMode==CopiedInFile) {
                                                //Copy in temporary file
                                                temporaryFile=QDir::tempPath()+"/skg_"+QUuid::createUuid ().toString()+".skg";
                                                currentDatabase=new QSqlDatabase(QSqlDatabase::addDatabase("QSQLITE", databaseIdentifier));
                                                currentDatabase->setDatabaseName(temporaryFile);
                                                if (!currentDatabase->open()) {
                                                        //Set error message
                                                        QSqlError sqlErr=currentDatabase->lastError();
                                                        err = SKGError(SQLLITEERROR + sqlErr.number(), sqlErr.text());
                                                }
                                        }

                                        //Create parameter and undo redo table
                                        /**
                                         * This constant is used to initialized the data model (table creation)
                                         */
                                        const QString InitialDataModel
                                        =
                                                // ==================================================================
                                                //Table parameters
                                                "DROP TABLE IF EXISTS parameters;;"
                                                "CREATE TABLE parameters "
                                                "(id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,"
                                                "t_uuid_parent TEXT NOT NULL DEFAULT '',"
                                                "t_name TEXT NOT NULL,"
                                                "t_value TEXT NOT NULL DEFAULT '',"
                                                "b_blob BLOB,"
                                                "d_lastmodifdate DATE NOT NULL DEFAULT CURRENT_TIMESTAMP);;"

                                                // ==================================================================
                                                //Table node
                                                "DROP TABLE IF EXISTS node;;"
                                                "CREATE TABLE node ("
                                                "id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,"
                                                "t_name TEXT NOT NULL DEFAULT '' CHECK (t_name NOT LIKE '%"OBJECTSEPARATOR"%'),"
                                                "t_fullname TEXT,"
                                                "f_sortorder FLOAT,"
                                                "t_autostart VARCHAR(1) DEFAULT 'N' CHECK (t_autostart IN ('Y', 'N')),"
                                                "t_data TEXT,"
                                                "r_node_id INT CONSTRAINT fk_id REFERENCES node(id) ON DELETE CASCADE);;"

                                                // ==================================================================
                                                //Table doctransaction
                                                "DROP TABLE IF EXISTS doctransaction;;"
                                                "CREATE TABLE doctransaction ("
                                                "id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,"
                                                "t_name TEXT NOT NULL,"
                                                "t_mode VARCHAR(1) DEFAULT 'U' CHECK (t_mode IN ('U', 'R')),"
                                                "d_date DATE NOT NULL,"
                                                "t_savestep VARCHAR(1) DEFAULT 'N' CHECK (t_savestep IN ('Y', 'N')),"
                                                "i_parent INTEGER);;"

                                                // ==================================================================
                                                //Table doctransactionitem
                                                "DROP TABLE IF EXISTS doctransactionitem;;"
                                                "CREATE TABLE doctransactionitem ("
                                                "id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, "
                                                "rd_doctransaction_id INTEGER NOT NULL,"
                                                "i_object_id INTEGER NOT NULL,"
                                                "t_object_table TEXT NOT NULL,"
                                                "t_action VARCHAR(1) DEFAULT 'I' CHECK (t_action IN ('I', 'U', 'D')),"
                                                "t_sqlorder TEXT NOT NULL DEFAULT '');;"

                                                "DROP TABLE IF EXISTS doctransactionmsg;;"
                                                "CREATE TABLE doctransactionmsg ("
                                                "id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, "
                                                "rd_doctransaction_id INTEGER NOT NULL,"
                                                "t_message TEXT NOT NULL DEFAULT '',"
                                                "t_popup VARCHAR(1) DEFAULT 'Y' CHECK (t_popup IN ('Y', 'N'))"
                                                ");;";
                                        ;

                                        if (err.isSucceeded()) err = SKGServices::executeSqliteOrder(this, InitialDataModel);
                                        if (err.isSucceeded()) err = SKGDocument::refreshViewsIndexesAndTriggers();
                                }
                        }
                }
        }

        //migrate
        if (err.isSucceeded()) {
                bool mig=false;
                err = migrate(mig);
                if (err.isSucceeded() && mig && !name.isEmpty()) err=sendMessage(i18n("The document has been migrated"));
        }

        //Optimization
        if (err.isSucceeded()) {
                lastSavedTransaction=getTransactionToProcess(SKGDocument::UNDO);
                SKGServices::executeSqliteOrder(this, "ANALYZE");
        }

        //Set case sensitive
        if (err.isSucceeded()) err = SKGServices::executeSqliteOrder(this, "PRAGMA case_sensitive_like=true");

        //Set locking_mode EXCLUSIVE
        if (err.isSucceeded()) err = SKGServices::executeSqliteOrder(this, "PRAGMA locking_mode=EXCLUSIVE");

        //For better performance
        if (err.isSucceeded() && dbMode==CopiedInFile) err = SKGServices::executeSqliteOrder(this, "PRAGMA synchronous = OFF");

        //Creation undo/redo triggers
        if (err.isSucceeded()) err = createUndoRedoTemporaryTriggers();

        if (err.isFailed() && !name.isEmpty()) {
                close();
        } else {
                //Send event
                emit tableModified("", 0);
        }

        return err;
}

bool SKGDocument::isFileModified() const
{
        //Get last executed transaction
        int last=getTransactionToProcess(SKGDocument::UNDO);
        //  if (nbStepForTransaction.size()) --last;
        return (lastSavedTransaction!=last);
}

QString SKGDocument::getCurrentFileName() const
{
        return currentFileName;
}

SKGError SKGDocument::save()
{
        SKGError err;
        SKGTRACEINRC(5, "SKGDocument::save", err);
        if (currentFileName.length() == 0) {
                err = SKGError(ERR_INVALIDARG, i18n("Save not authorized because the file name is not yet defined"));
        } else {
                //save
                err = saveAs(currentFileName, true);
        }
        return err;
}

SKGError SKGDocument::saveAs(const QString & name, bool overwrite)
{
        SKGError err;
        SKGTRACEINRC(5, "SKGDocument::saveAs", err);
        SKGTRACEL(10) << "Input parameter [name]=[" << name << ']' << endl;

        //Check if a transaction is still opened
        err = checkExistingTransaction();
        if (err.isSucceeded()) {
                //A transaction is still opened
                err.setReturnCode(ERR_ABORT);
                err.setMessage(i18n("Save is forbidden if a transaction is still opened"));

        } else {
                err=SKGError();

                if (getParameter("SKG_UNDO_CLEAN_AFTER_SAVE")=="Y") {
                        err = SKGServices::executeSqliteOrder(this,"delete from doctransaction");
                }

                //No transaction opened ==> it's ok
                //We mark the last transaction as a save point
                if (err.isSucceeded()) {
                        err = SKGServices::executeSqliteOrder(this,"update doctransaction set t_savestep='Y' where id in (select A.id from doctransaction A where "
                                                              "((select count(1) from doctransaction B where B.i_parent=A.id)=0) "
                                                              "and A.t_mode='U')");
                }
                emit tableModified("doctransaction", 0);

                //Optimization
                if (err.isSucceeded()) {
                        DatabaseMode dbMode=getDatabaseMode();

                        if (dbMode!=SKGDocument::CopiedInMemory) err = SKGServices::executeSqliteOrder(this, "VACUUM;");
                        if (err.isSucceeded()) {
                                //Check if file already exist
                                if (!overwrite && QFile(name).exists()) {
                                        //Set error message
                                        err.setReturnCode(ERR_INVALIDARG);
                                        err.setMessage(i18n("File [%1] already exist",name));
                                } else {
                                        //Get backup file name
                                        bool backupFileMustBeRemoved=false;
                                        QString backupFileName=getBackupFile(name);
                                        if (backupFileName.isEmpty()) {
                                                backupFileName=name+".tmp";
                                                backupFileMustBeRemoved=true;
                                        }

                                        //Create backup file
                                        QFile::remove(backupFileName);
                                        if (QFile(name).exists() && !QFile(name).copy(backupFileName)) {
                                                err = SKGError(ERR_FAIL, i18n("Creation of backup file %1 failed",backupFileName));
                                        }

                                        //Open database
                                        if (err.isSucceeded()) {
                                                if (dbMode==CopiedInMemory) {
                                                        QFile::remove(name);

                                                        {//This scope is needed to avoid a warning at remove
                                                                if (QFile(name).exists()) {
                                                                        //Set error message
                                                                        err.setReturnCode(ERR_FAIL);
                                                                        err.setMessage(i18n("Impossible to remove [%1]",name));
                                                                } else {
                                                                        //Copy memory in file
                                                                        QSqlDatabase dbfile = QSqlDatabase::addDatabase("QSQLITE", "TEMPORARY_SAVE");
                                                                        dbfile.setDatabaseName(name);
                                                                        if (!dbfile.open()) {
                                                                                //Set error message
                                                                                QSqlError sqlErr=dbfile.lastError();
                                                                                err = SKGError(SQLLITEERROR + sqlErr.number(), sqlErr.text());
                                                                        } else {
                                                                                //Copy db from memory to file
                                                                                err = SKGServices::copySqliteDatabase(&dbfile, getDatabase(), false);

                                                                                //Close db
                                                                                dbfile.close();
                                                                        }
                                                                }
                                                        }

                                                        //Remove database
                                                        QSqlDatabase::removeDatabase("TEMPORARY_SAVE");
                                                } else if (dbMode==CopiedInFile) {
                                                        QFile::remove(name);

                                                        //To be sure that db is flushed
                                                        if (err.isSucceeded()) err=SKGServices::executeSqliteOrder(this, "PRAGMA synchronous = FULL");

                                                        //Copy file to file
                                                        if (err.isSucceeded()) err=SKGServices::cryptFile(temporaryFile, name, getParameter("SKG_PASSWORD"), true);

                                                        //For performances
                                                        if (err.isSucceeded()) err=SKGServices::executeSqliteOrder(this, "PRAGMA synchronous = OFF");

                                                        //Restore backup in case of failure
                                                        if (err.isFailed()) {
                                                                QFile::remove(name);
                                                                QFile(backupFileName).rename(name);
                                                        }
                                                } else if (dbMode==DirectAccess && !temporaryFile.isEmpty()) {
                                                        QFile::remove(name);

                                                        //Copy file to file
                                                        if (!QFile::copy(temporaryFile, name)) {
                                                                err = SKGError(ERR_FAIL, i18n("copy of %1 to %2 failed",temporaryFile,name));
                                                        } else {
                                                                err=close();
                                                                if (err.isSucceeded()) err=load(name);
                                                        }

                                                        //Restore backup in case of failure
                                                        if (err.isFailed()) {
                                                                QFile::remove(name);
                                                                QFile(backupFileName).rename(name);
                                                        }
                                                }
                                        }

                                        if (backupFileMustBeRemoved) QFile::remove(backupFileName);

                                        if (err.isSucceeded()) {
                                                //The document is not modified
                                                currentFileName = name;
                                                lastSavedTransaction=getTransactionToProcess(SKGDocument::UNDO);
                                        }
                                }
                        }
                }
        }
        return err;
}

SKGError SKGDocument::close()
{
        SKGError err;
        SKGTRACEINRC(5, "SKGDocument::close", err);
        if (getDatabase() != NULL) {
                getDatabase()->close();
                delete currentDatabase; //Delete handler to avoid warning and memory leak
                QSqlDatabase::removeDatabase(databaseIdentifier);
        }

        if (!temporaryFile.isEmpty()) {
                QFile(temporaryFile).remove();
                temporaryFile="";
        }

        //Emit events ?
        bool emitEvent=(lastSavedTransaction!=-1);

        //Init fields
        currentDatabase = NULL;;
        currentFileName = "";
        lastSavedTransaction=0;
        nbStepForTransaction=SKGIntList();
        posStepForTransaction=SKGIntList();

        //Send event
        if (emitEvent) {
                emit tableModified("", 0);
                emit transactionSuccessfullyEnded(0);
        }

        return err;
}

SKGError SKGDocument::refreshViewsIndexesAndTriggers() const
{
        SKGError err;
        SKGTRACEINRC(5, "SKGDocument::refreshViewsIndexesAndTriggers", err);
        /**
        * This constant is used to initialized the data model (trigger creation)
        */
        const QString InitialDataModelTrigger
        =
                DELETECASCADEPARAMETER("parameters")+
                DELETECASCADEPARAMETER("node")+

                //Compute fullname
                "DROP TRIGGER IF EXISTS cpt_node_fullname1;;"
                "CREATE TRIGGER cpt_node_fullname1 " //This trigger must be the first
                "AFTER UPDATE OF t_fullname ON node BEGIN "
                "UPDATE node SET t_name=t_name WHERE r_node_id=new.id;"
                "END;;"

                "DROP TRIGGER IF EXISTS cpt_node_fullname2;;"
                "CREATE TRIGGER cpt_node_fullname2 "
                "AFTER INSERT ON node BEGIN "
                "UPDATE node SET t_fullname="
                "CASE WHEN r_node_id IS NULL THEN new.t_name ELSE (SELECT c.t_fullname from node c where c.id=new.r_node_id)||'"OBJECTSEPARATOR"'||new.t_name END "
                "WHERE id=new.id;"
                "END;;"

                "DROP TRIGGER IF EXISTS cpt_node_fullname3;;"
                "CREATE TRIGGER cpt_node_fullname3 "
                "AFTER UPDATE OF t_name ON node BEGIN "
                "UPDATE node SET t_fullname="
                "CASE WHEN r_node_id IS NULL OR r_node_id='' THEN new.t_name ELSE (SELECT c.t_fullname from node c where c.id=new.r_node_id)||'"OBJECTSEPARATOR"'||new.t_name END "
                "WHERE id=new.id;"
                "END;;"

                //-- Foreign Key Preventing insert
                /*   "
                CREATE TRIGGER fki_node_parent_id_node_id "
                "BEFORE INSERT ON [node] "
                "FOR EACH ROW BEGIN "
                "  SELECT RAISE(ABORT, 'insert on table node violates foreign key constraint fki_node_parent_id_node_id') "
                "  WHERE NEW.r_node_id IS NOT NULL AND NEW.r_node_id!='' AND (SELECT id FROM node WHERE id = NEW.r_node_id) IS NULL; "
                "
                END;; "

                //-- Foreign key preventing update
                "
                CREATE TRIGGER fku_node_parent_id_node_id "
                "BEFORE UPDATE ON [node] "
                "FOR EACH ROW BEGIN "
                "    SELECT RAISE(ABORT, 'update on table node violates foreign key constraint fku_node_parent_id_node_id') "
                "      WHERE NEW.r_node_id IS NOT NULL AND NEW.r_node_id!='' AND (SELECT id FROM node WHERE id = NEW.r_node_id) IS NULL; "
                "
                END;; "
                */ //TODO


                //-- Cascading delete - WARNING rewriten to support recursive mode
                "DROP TRIGGER IF EXISTS fkdc_node_parent_id_node_id;;"
                "CREATE TRIGGER fkdc_node_parent_id_node_id "
                "BEFORE DELETE ON node "
                "FOR EACH ROW BEGIN "
                "    DELETE FROM node WHERE node.t_fullname LIKE OLD.t_fullname||'"OBJECTSEPARATOR"%'; "
                "END;; "

                /*     "
                CREATE TRIGGER fkdc_node_parent_id_node_id "
                "BEFORE DELETE ON node "
                "FOR EACH ROW BEGIN "
                "    DELETE FROM node WHERE node.r_node_id=OLD.id; "
                "
                END;; "*/
                ;

        /**
        * This constant is used to initialized the data model (index creation)
        */
        const QString InitialDataModelIndex
        =
                "DROP INDEX IF EXISTS uidx_parameters_uuid_parent_name;;"
                "CREATE UNIQUE INDEX uidx_parameters_uuid_parent_name ON parameters (t_uuid_parent, t_name);;"

                "DROP INDEX IF EXISTS uidx_node_parent_id_name;;"
                "CREATE UNIQUE INDEX uidx_node_parent_id_name ON node(t_name,r_node_id);;"

                "DROP INDEX IF EXISTS uidx_node_fullname;;"
                "CREATE UNIQUE INDEX uidx_node_fullname ON node(t_fullname);;"

                "DROP INDEX IF EXISTS idx_doctransaction_parent;;"
                "CREATE INDEX idx_doctransaction_parent ON doctransaction (i_parent);;"

                "DROP INDEX IF EXISTS idx_doctransactionitem_i_object_id;;"
                "CREATE INDEX idx_doctransactionitem_i_object_id ON doctransactionitem (i_object_id);;"

                "DROP INDEX IF EXISTS idx_doctransactionitem_t_object_table;;"
                "CREATE INDEX idx_doctransactionitem_t_object_table ON doctransactionitem (t_object_table);;"

                "DROP INDEX IF EXISTS idx_doctransactionitem_t_action;;"
                "CREATE INDEX idx_doctransactionitem_t_action ON doctransactionitem (t_action);;"

                "DROP INDEX IF EXISTS idx_doctransactionitem_rd_doctransaction_id;;"
                "CREATE INDEX idx_doctransactionitem_rd_doctransaction_id ON doctransactionitem (rd_doctransaction_id);;"

                "DROP INDEX IF EXISTS idx_doctransactionitem_optimization;;"
                "CREATE INDEX idx_doctransactionitem_optimization ON doctransactionitem (rd_doctransaction_id, i_object_id, t_object_table, t_action, id);;"
                ;

        /**
        * This constant is used to initialized the data model (view creation)
        */
        const QString InitialDataModelView
        =
                "DROP VIEW IF EXISTS v_node;;"
                "CREATE VIEW  v_node AS SELECT * from node;;"

                "DROP VIEW IF EXISTS v_node_displayname;;"
                "CREATE VIEW v_node_displayname AS SELECT *, t_fullname AS t_displayname from node;;"

                "DROP VIEW IF EXISTS v_parameters_displayname;;"
                "CREATE VIEW v_parameters_displayname AS SELECT *, t_name AS t_displayname from parameters;;"
                ;

        err = SKGServices::executeSqliteOrder(this, InitialDataModelIndex);
        if (err.isSucceeded()) err = SKGServices::executeSqliteOrder(this, InitialDataModelView);
        if (err.isSucceeded()) err = SKGServices::executeSqliteOrder(this, InitialDataModelTrigger);
        if (err.isSucceeded()) {
                //Refresh dynamic triggers
                QRegExp rx_rd("rd_([^_]+)_([^_]+).*");
                QRegExp rx_rc("rc_([^_]+)_([^_]+).*");
                QStringList tables;
                err=SKGServices::getDistinctValues(this, "sqlite_master", "name", "type='table'", tables);
                int nb=tables.count();
                for (int i=0; err.isSucceeded() && i<nb; ++i) {
                        QString table=tables[i];
                        SKGStringListList attributes;
                        err=SKGServices::executeSelectSqliteOrder(this, "PRAGMA table_info("+table+");", attributes);
                        int nb2=attributes.count();
                        for (int j=1; err.isSucceeded() && j<nb2; ++j) { //Header is ignored
                                QString att=attributes.at(j).at(1);
                                if (rx_rd.indexIn(att)!=-1) {
                                        //Get parameters
                                        QString tab2=rx_rd.cap(1);
                                        QString att2 =rx_rd.cap(2);
                                        err = SKGServices::executeSqliteOrder(this, FOREIGNCONSTRAINTCASCADE(tab2,att2,table,att));
                                } else if (rx_rc.indexIn(att)!=-1) {
                                        //Get parameters
                                        QString tab2=rx_rc.cap(1);
                                        QString att2 =rx_rc.cap(2);
                                        err = SKGServices::executeSqliteOrder(this, FOREIGNCONSTRAINT(tab2,att2,table,att));
                                }
                        }
                }
        }

        return err;
}

SKGError SKGDocument::migrate(bool& oMigrationDone)
{
        SKGError err;
        SKGTRACEINRC(5, "SKGDocument::migrate", err);
        oMigrationDone=false;

        err = beginTransaction("#INTERNAL#");
        if (getParameter("SKG_UNDO_MAX_DEPTH").isEmpty()) {
                err = setParameter("SKG_UNDO_MAX_DEPTH", SKGServices::intToString(SKG_UNDO_MAX_DEPTH));
        }

        if (getParameter("SKG_UNDO_CLEAN_AFTER_SAVE").isEmpty()) {
                err = setParameter("SKG_UNDO_CLEAN_AFTER_SAVE", "N");
        }

        uniqueIdentifier=getParameter("SKG_UNIQUE_ID");
        if (getParameter("SKG_UNIQUE_ID").isEmpty()) {
                uniqueIdentifier=QUuid::createUuid ().toString();
                err = setParameter("SKG_UNIQUE_ID", uniqueIdentifier);
        }

        QString version = getParameter("SKG_DB_VERSION");
        QString initialversion = version;

        if (err.isSucceeded() && version.isEmpty()) {
                //First creation
                SKGTRACEL(10) << "Migration from 0 to 0.8" << endl;

                //Set new version
                version = "0.8";
                if (err.isSucceeded()) err = setParameter("SKG_DB_VERSION", version);

                //Set sqlite creation version
                SKGStringListList listTmp;
                if (err.isSucceeded()) err=SKGServices::executeSelectSqliteOrder(this, "select sqlite_version()", listTmp);
                if (err.isSucceeded() && listTmp.count()==2) err = setParameter("SKG_SQLITE_CREATION_VERSION", listTmp.at(1).at(0));
                oMigrationDone=true;
        }

        if (err.isSucceeded() && version == "0.1") {
                //Migration from version 0.1 to 0.2
                SKGTRACEL(10) << "Migration from 0.1 to 0.2" <<endl;

                // ==================================================================
                //Table doctransactionmsg
                QString sql=
                        "DROP TABLE IF EXISTS doctransactionmsg;;"
                        "CREATE TABLE doctransactionmsg ("
                        "id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, "
                        "rd_doctransaction_id INTEGER NOT NULL,"
                        "t_message TEXT NOT NULL DEFAULT '');;";
                err = SKGServices::executeSqliteOrder(this, sql);

                //Set new version
                version="0.2";
                if (err.isSucceeded()) err=SKGDocument::setParameter ("SKG_DB_VERSION", version);
                oMigrationDone=true;
        }

        if (err.isSucceeded() && version == "0.2") {
                //Migration from version 0.2 to 0.3
                SKGTRACEL(10) << "Migration from 0.2 to 0.3" <<endl;

                err = SKGServices::executeSqliteOrder(this, "UPDATE node set f_sortorder=id");

                //Set new version
                version="0.3";
                if (err.isSucceeded()) err=SKGDocument::setParameter ("SKG_DB_VERSION", version);
                oMigrationDone=true;
        }

        if (err.isSucceeded() && version == "0.3") {
                //Migration from version 0.3 to 0.4
                SKGTRACEL(10) << "Migration from 0.3 to 0.4" <<endl;

                err = SKGServices::executeSqliteOrder(this, "ALTER TABLE node ADD COLUMN t_autostart VARCHAR(1) DEFAULT 'N' CHECK (t_autostart IN ('Y', 'N'))");
                if (err.isSucceeded()) err=SKGServices::executeSqliteOrder (this, "UPDATE node set t_autostart='Y' where t_name='"+i18nc("Verb, automatically load when the application is started","autostart")+'\'');

                //Set new version
                version="0.4";
                if (err.isSucceeded()) err=SKGDocument::setParameter ("SKG_DB_VERSION", version);
                oMigrationDone=true;
        }
        if (err.isSucceeded() && version == "0.4") {
                //Migration from version 0.4 to 0.5
                SKGTRACEL(10) << "Migration from 0.4 to 0.5" <<endl;

                err = SKGServices::executeSqliteOrder(this, "ALTER TABLE doctransactionmsg ADD COLUMN t_popup VARCHAR(1) DEFAULT 'Y' CHECK (t_popup IN ('Y', 'N'))");

                //Set new version
                version="0.5";
                if (err.isSucceeded()) err=SKGDocument::setParameter ("SKG_DB_VERSION", version);
                oMigrationDone=true;
        }
        if (err.isSucceeded() && version == "0.5") {
                //Migration from version 0.5 to 0.6
                SKGTRACEL(10) << "Migration from 0.5 to 0.6" <<endl;

                err=SKGServices::executeSqliteOrder (this, "UPDATE node set t_autostart='N' where t_autostart NOT IN ('Y', 'N')");

                //Set new version
                version="0.6";
                if (err.isSucceeded()) err=SKGDocument::setParameter ("SKG_DB_VERSION", version);
                oMigrationDone=true;
        }
        if (err.isSucceeded() && version == "0.6") {
                //Migration from version 0.6 to 0.7
                SKGTRACEL(10) << "Migration from 0.6 to 0.7" <<endl;

                err = SKGServices::executeSqliteOrder(this, "ALTER TABLE parameters ADD COLUMN b_blob BLOB");

                //Set new version
                version="0.7";
                if (err.isSucceeded()) err=SKGDocument::setParameter ("SKG_DB_VERSION", version);
                oMigrationDone=true;
        }
        if (err.isSucceeded() && version == "0.7") {
                //Migration from version 0.7 to 0.8
                SKGTRACEL(10) << "Migration from 0.7 to 0.8" <<endl;

                err = SKGServices::executeSqliteOrder(this, "UPDATE parameters set t_name='SKG_LANGUAGE' where t_name='SKGLANGUAGE'");

                //Set new version
                version="0.8";
                if (err.isSucceeded()) err=SKGDocument::setParameter ("SKG_DB_VERSION", version);
                oMigrationDone=true;
        }
        //Refresh views
        if (err.isSucceeded())  err = refreshViewsIndexesAndTriggers();

        //Set sqlite last version
        SKGStringListList listTmp;
        if (err.isSucceeded()) err=SKGServices::executeSelectSqliteOrder(this, "select sqlite_version()", listTmp);
        if (err.isSucceeded() && listTmp.count()==2) err = setParameter("SKG_SQLITE_LAST_VERSION", listTmp.at(1).at(0));


        if (err.isFailed()) err.addError(ERR_FAIL, i18n("Database migration from version %1 to version %2 failed",initialversion,version));

        //close temporary transaction
        if (err.isSucceeded()) err=endTransaction(true);
        else  endTransaction(false);

        return err;
}

SKGError SKGDocument::createUndoRedoTemporaryTriggers() const
{
        SKGError err;
        SKGTRACEINRC(10, "SKGDocument::createUndoRedoTemporaryTriggers", err);

        //Create triggers
        QStringList tables;
        err=SKGServices::getTablesList(this, tables);
        int nbTables=tables.count();
        for (int i=0; err.isSucceeded() && i<nbTables; ++i) {
                //Get table name
                QString table = tables[i];

                //Do we have to treat this table
                if (!SKGListNotUndoable.contains("T."+table)) {
                        //YES
                        //Get attributes name
                        QStringList attributes;
                        err=SKGServices::getAttributesList(this, table, attributes);

                        //Build sqlorder for update and insert
                        QString sqlorderForUpdate2;
                        QString sqlorderForInsert1;
                        QString sqlorderForInsert2;
                        int nbAttributes=attributes.count();
                        for (int j=0; err.isSucceeded() && j<nbAttributes; ++j) {
                                //Get attribute
                                QString att = attributes[j];

                                //Do we have to treat this attribute
                                if (!SKGListNotUndoable.contains("A."+table+'.'+att)) {

                                        //Build for update
                                        if (sqlorderForUpdate2.length() > 0)
                                                sqlorderForUpdate2 += ',';
                                        sqlorderForUpdate2 += att + "='||quote(old." + att + ")||'";

                                        //Build for insert part 1
                                        if (sqlorderForInsert1.length() > 0)
                                                sqlorderForInsert1 += ',';
                                        sqlorderForInsert1 += att;

                                        //Build for insert part 2
                                        if (sqlorderForInsert2.length() > 0)
                                                sqlorderForInsert2 += ',';
                                        sqlorderForInsert2 += "'||quote(old." + att + ")||'";
                                }
                        }

                        //Create specific triggers for the current transaction
                        err = SKGServices::executeSqliteOrder(this,
                                                              //DROP DELETE trigger
                                                              "DROP TRIGGER IF EXISTS UR_" + table + "_IN;;"

                                                              //Create DELETE trigger
                                                              "CREATE TEMP TRIGGER UR_" + table + "_IN "
                                                              "AFTER  INSERT ON " + table + " BEGIN "
                                                              "INSERT INTO doctransactionitem (rd_doctransaction_id, t_sqlorder,i_object_id,t_object_table,t_action) VALUES(0,'DELETE FROM " + table +
                                                              " WHERE id='||new.id,new.id,'" + table + "','D');END;;"

                                                              //DROP UPDATE trigger
                                                              "DROP TRIGGER IF EXISTS UR_" + table + "_UP;;"

                                                              //Create UPDATE trigger
                                                              "CREATE TEMP TRIGGER UR_" + table + "_UP "
                                                              "AFTER UPDATE ON " + table + " BEGIN "
                                                              "INSERT INTO doctransactionitem  (rd_doctransaction_id, t_sqlorder,i_object_id,t_object_table,t_action) VALUES(0,'UPDATE " + table +
                                                              " SET " + sqlorderForUpdate2 +
                                                              " WHERE id='||new.id,new.id,'" + table + "','U');END;;"

                                                              //DROP INSERT trigger
                                                              "DROP TRIGGER IF EXISTS UR_" + table + "_DE;;"

                                                              //Create INSERT trigger
                                                              "CREATE TEMP TRIGGER UR_" + table + "_DE "
                                                              "AFTER DELETE ON " + table +
                                                              " BEGIN "
                                                              "INSERT INTO doctransactionitem  (rd_doctransaction_id, t_sqlorder,i_object_id,t_object_table,t_action) VALUES(0,'INSERT INTO " + table +
                                                              '(' + sqlorderForInsert1 + ") VALUES(" + sqlorderForInsert2 + ")',old.id,'" + table + "','I'); END;;");
                }
        }
        return err;
}

QString SKGDocument::getParameter(const QString& iName, const QString& iParentUUID) const
{
        SKGTRACEIN(10, "SKGDocument::getParameter");
        SKGTRACEL(10) << "Input parameter [iName]=[" << iName << ']' << endl;
        QString output;

        //Get parameter
        SKGObjectBase param;
        SKGError err = SKGObjectBase::getObject(this, "parameters", "t_name='"+SKGServices::stringToSqlString(iName)+
                                                "' AND t_uuid_parent='"+SKGServices::stringToSqlString(iParentUUID)+'\'', param);
        if (err.isSucceeded()) {
                output = param.getAttribute("t_value");
        }
        return output;
}

QVariant SKGDocument::getParameterBlob(const QString& iName, const QString& iParentUUID) const
{
        SKGTRACEIN(10, "SKGDocument::getParameterBlob");
        SKGTRACEL(10) << "Input parameter [iName]=[" << iName << ']' << endl;
        QVariant output;

        QString sqlQuery="SELECT b_blob FROM parameters WHERE t_name=? AND t_uuid_parent=?";
        QSqlQuery query(*getDatabase());
        query.prepare(sqlQuery);
        query.addBindValue(iName);
        query.addBindValue(iParentUUID);
        if (!query.exec()) {
                QSqlError sqlError=query.lastError();
                SKGTRACE << "WARNING: " << sqlQuery << endl;
                SKGTRACE << "         returns :" << sqlError.text() << endl;
        } else {
                if (query.next()) output=query.value(0);
        }

        return output;
}

SKGError SKGDocument::setParameter(const QString& iName, const QString& iValue, const QVariant& iBlob, const QString& iParentUUID, SKGObjectBase* oObjectCreated) const
{
        SKGError err;
        SKGTRACEINRC(10, "SKGDocument::setParameter", err);
        SKGTRACEL(10) << "Input parameter [iName]=[" << iName << ']' << endl;
        SKGTRACEL(10) << "Input parameter [iValue]=[" << iValue << ']' << endl;

        SKGNamedObject param(this, "parameters");
        if (err.isSucceeded()) err = param.setName(iName);
        if (err.isSucceeded()) err = param.setAttribute("t_value", iValue);
        if (err.isSucceeded()) err = param.setAttribute("t_uuid_parent", iParentUUID);
        if (err.isSucceeded()) err = param.save();

        if (err.isSucceeded() && !iBlob.isNull()) {
                err = param.load();
                if (err.isSucceeded()) {
                        //Set blob
                        QString sqlQuery="UPDATE parameters SET b_blob=? WHERE id=?";
                        QSqlQuery query(*getDatabase());
                        query.prepare(sqlQuery);
                        query.addBindValue(iBlob);
                        query.addBindValue(param.getID());
                        if (!query.exec()) {
                                QSqlError sqlError=query.lastError();

                                QString msg=sqlQuery+':'+sqlError.text();
                                err=SKGError(SQLLITEERROR + sqlError.number(), msg);

                                QVariant v = getDatabase()->driver()->handle();
                                if (v.isValid() && qstrcmp(v.typeName(), "sqlite3*")==0) {
                                        sqlite3* handle = *static_cast<sqlite3 **>(v.data());
                                        if (handle != NULL) { // check that it is not NULL
                                                QString dmsg=sqlite3_errmsg(handle);
                                                if (!dmsg.isEmpty()) err.addError(SQLLITEERROR + sqlError.number(), dmsg);
                                        }
                                }
                        } else {
                                if (oObjectCreated!=NULL) *oObjectCreated=param;
                        }
                }
        }

        return err;
}

SKGError SKGDocument::dump(int iMode) const
{
        SKGError err;
        if (getDatabase()) {
                //dump parameters
                SKGTRACE << "=== START DUMP ===" << endl;
                if (iMode&DUMPSQLITE) {
                        SKGTRACE << "=== DUMPSQLITE ===" << endl;
                        err.addError(SKGServices::dumpSelectSqliteOrder(this, "SELECT * FROM sqlite_master order by type"));

                        SKGTRACE << "=== DUMPSQLITE (TEMPORARY) ===" << endl;
                        err.addError(SKGServices::dumpSelectSqliteOrder(this, "SELECT * FROM sqlite_temp_master order by type"));
                }

                if (iMode&DUMPPARAMETERS) {
                        SKGTRACE << "=== DUMPPARAMETERS ===" << endl;
                        err.addError(SKGServices::dumpSelectSqliteOrder(this, "SELECT * FROM parameters order by id"));
                }

                if (iMode&DUMPNODES) {
                        SKGTRACE << "=== DUMPNODES ===" << endl;
                        err.addError(SKGServices::dumpSelectSqliteOrder(this, "SELECT * FROM node order by id"));
                }

                if (iMode&DUMPTRANSACTIONS) {
                        //dump transaction
                        SKGTRACE << "=== DUMPTRANSACTIONS ===" << endl;
                        err.addError(SKGServices::dumpSelectSqliteOrder(this, "SELECT * FROM doctransaction order by id"));

                        //dump transaction
                        SKGTRACE << "=== DUMPTRANSACTIONS (ITEMS) ===" << endl;
                        err.addError(SKGServices::dumpSelectSqliteOrder(this, "SELECT * FROM doctransactionitem order by rd_doctransaction_id, id"));
                }
                SKGTRACE << "=== END DUMP ===" << endl;
        }
        return err;
}

QSqlDatabase* SKGDocument::getDatabase() const
{
        return currentDatabase;
}

SKGError SKGDocument::getConsolidatedView(const QString& iTable,
                const QString& iAsColumn,
                const QString& iAsRow,
                const QString& iAttribute,
                const QString& iOpAtt,
                const QString& iWhereClause,
                SKGStringListList& oTable,
		const QString& iMissingValue) const
{
        SKGError err;
        SKGTRACEINRC(10, "SKGDocument::getConsolidatedView", err);
        SKGTRACEL(10) << "Input parameter [iTable]=[" << iTable << ']' << endl;
        SKGTRACEL(10) << "Input parameter [iAsColumn]=[" << iAsColumn << ']' << endl;
        SKGTRACEL(10) << "Input parameter [iAsRow]=[" << iAsRow << ']' << endl;
        SKGTRACEL(10) << "Input parameter [iAttribute]=[" << iAttribute << ']' << endl;
        SKGTRACEL(10) << "Input parameter [iOpAtt]=[" << iOpAtt << ']' << endl;
        SKGTRACEL(10) << "Input parameter [iWhereClause]=[" << iWhereClause << ']' << endl;
        SKGTRACEL(10) << "Input parameter [iMissingValue]=[" << iMissingValue << ']' << endl;

        //Mode
        int mode=0;
        if (!iAsColumn.isEmpty()) mode+=1;
        if (!iAsRow.isEmpty()) mode+=2;

        oTable.clear();
        oTable.push_back(QStringList());

        QStringList* titles=(QStringList*) &(oTable.at(0));

        if (mode==3) {
                titles->push_back(iAsRow+'/'+iAsColumn);
        } else {
                if (mode==1) {
                        titles->push_back(iAsColumn);

                        QStringList sums;
                        sums.push_back(i18nc("Noun, the numerical sum of a list of values","Sum"));
                        oTable.push_back(sums);
                } else {
                        if (mode==2) {
                                titles->push_back(iAsRow);
                                titles->push_back(i18nc("Noun, the numerical sum of a list of values","Sum"));
                        }
                }
        }

        //Create sqlorder
        QString att=iAsColumn;
        if (!att.isEmpty() && !iAsRow.isEmpty()) att+=',';
        att+=iAsRow;

        QString sort=iAsRow;
        if (!sort.isEmpty() && !iAsColumn.isEmpty()) sort+=',';
        sort+=iAsColumn;

        if (!att.isEmpty()) {
                QString sql="SELECT "+att+','+iOpAtt+'('+iAttribute+") FROM "+iTable;
                if (!iWhereClause.isEmpty()) sql+=" WHERE "+iWhereClause;
                if (!iOpAtt.isEmpty()) sql+=" GROUP BY "+att;
                sql+=" ORDER BY "+sort;

                QHash<QString, int> cols;
                QHash<QString, int> rows;

                SKGTRACEL(10) << "sqlorder=[" << sql << ']' << endl;
                SKGStringListList listTmp;
                err=SKGServices::executeSelectSqliteOrder(this, sql, listTmp);
                int nb=listTmp.count();
                for (int i=1; err.isSucceeded() && i<nb; ++i) { //Title is ignored
                        QStringList line=listTmp.at(i);
                        int rowindex=-1;
                        int colindex=-1;
                        if (mode>=2) {
                                QString rowname=line.at(mode==3 ? 1 : 0);

                                if (!rows.contains(rowname)) {
                                        QStringList r;
                                        r.push_back(rowname);
                                        int nbx=oTable[0].count();
                                        for (int j=1; j<nbx; ++j)
                                                r.push_back(iMissingValue);

                                        oTable.push_back(r);

                                        rowindex=oTable.count()-1;
                                        rows.insert(rowname,rowindex);
                                } else rowindex=rows[rowname];
                        } else rowindex=1;

                        if (mode==1 || mode==3) {
                                QString colname=line.at(0);

                                if (!cols.contains(colname)) {
                                        //Search better position of this column
                                        colindex=-1;
                                        {
                                                QHashIterator<QString, int> cols_i(cols);
                                                while (cols_i.hasNext()) {
                                                        cols_i.next();
                                                        if (colname>cols_i.key() && cols_i.value()>colindex) colindex=cols_i.value();
                                                }
                                        }
                                        if (colindex==-1) colindex=1;
                                        else ++colindex;

                                        int nbx=oTable.count();
                                        for (int j=0; j<nbx; ++j) {
                                                if (j==0) oTable[j].insert(colindex,colname);
                                                else oTable[j].insert(colindex, iMissingValue);
                                        }

                                        {
                                                QHash<QString, int> tmp;
                                                QHashIterator<QString, int> cols_i(cols);
                                                while (cols_i.hasNext()) {
                                                        cols_i.next();
                                                        tmp.insert(cols_i.key(), cols_i.value()+(cols_i.value()>=colindex ? 1 : 0));
                                                }

                                                cols=tmp;
                                        }
                                        cols.insert(colname,colindex);

                                } else colindex=cols[colname];
                        } else colindex=1;

                        QString sum=line.at(mode==3 ? 2 : 1);

                        oTable[rowindex][colindex]=sum;
                }

                //Correction bug 205466 vvv
                //If some months or years are missing, we must add them.
                if (iAsColumn.startsWith(QLatin1String("d_"))) {
                        for (int c=1; c<oTable[0].count()-1; ++c) { //Dynamic size
                                QString dateFormat=(iAsColumn=="d_date" ? "yyyy-MM-dd" : (iAsColumn=="d_DATEMONTH" ? "yyyy-MM" : (iAsColumn=="d_DATEQUARTER" ? "yyyy-QM" : (iAsColumn=="d_DATESEMESTER" ? "yyyy-SM" : (iAsColumn=="d_DATEWEEK" ? "yyyy-WM" : "yyyy")))));
                                QDate nextExpected=QDate::fromString(oTable.at(0).at(c), dateFormat);
				QString nextExpectedString;
                                if (iAsColumn=="d_DATEWEEK")
				{
/* TODO
				  QStringList items=SKGServices::splitCSVLine(oTable.at(0).at(c),'-');
				  nextExpected=QDate(SKGServices::stringToInt(items.at(0)), 1, 1);
				  QString w=items.at(1);
				  w.remove('W');
				  nextExpected=nextExpected.addDays(7*SKGServices::stringToInt(w));
				  QString newW=SKGServices::intToString(nextExpected.weekNumber());
				  if(newW.count()==1) newW='0'+newW;
				  */
				  nextExpectedString=oTable.at(0).at(c+1);
				}
                                else if (iAsColumn=="d_DATEMONTH")
				{
				  nextExpected=nextExpected.addMonths(1);
				  nextExpectedString=nextExpected.toString (dateFormat);
				}
				else if (iAsColumn=="d_DATEQUARTER")
				{
				  nextExpected=nextExpected.addMonths(nextExpected.month()*3-nextExpected.month()); //convert quater in month
				  nextExpected=nextExpected.addMonths(3);
				  nextExpectedString=nextExpected.toString ("yyyy-Q")+(nextExpected.month()<=3 ? '1': (nextExpected.month()<=6 ? '2': (nextExpected.month()<=9 ? '3': '4')));
				}
                                else if (iAsColumn=="d_DATESEMESTER")
				{
				  nextExpected=nextExpected.addMonths(nextExpected.month()*6-nextExpected.month()); //convert semester in month
				  nextExpected=nextExpected.addMonths(6);
				  nextExpectedString=nextExpected.toString ("yyyy-S")+(nextExpected.month()<=6 ? '1' : '2');
				}
                                else if (iAsColumn=="d_DATEYEAR")
				{
				  nextExpected=nextExpected.addYears(1);
				  nextExpectedString=nextExpected.toString (dateFormat);
				}
				else
				{
				  nextExpected=nextExpected.addDays(1);
				  nextExpectedString=nextExpected.toString (dateFormat);
				}
                                if (oTable.at(0).at(c)!="0000" && oTable.at(0).at(c+1)!=nextExpectedString) {
                                        int colindex=c+1;

                                        int nbx=oTable.count();
					oTable[0].insert(colindex, nextExpectedString);
                                        for (int j=1; j<nbx; ++j) {
                                                oTable[j].insert(colindex, iMissingValue);
                                        }
                                }
                        }
                }
                //Correction bug 205466 ^^^
        }

        return err;
}

QList<SKGDocument::SKGModelTemplate> SKGDocument::getDisplaySchemas(const QString& iRealTable) const
{
        QList<SKGDocument::SKGModelTemplate> listSchema;

        //Build schemas
        if (iRealTable=="doctransaction") {
                SKGModelTemplate def;
                def.id="default";
                def.name=i18nc("Noun, the default value of an item","Default");
                def.icon="edit-undo";
                def.schema="t_name;t_value;d_lastmodifdate;t_savestep";
                listSchema.push_back(def);

                SKGModelTemplate minimum;
                minimum.id="minimum";
                minimum.name=i18nc("Noun, the minimum value of an item","Minimum");
                minimum.icon="";
                minimum.schema="t_name;t_value;d_lastmodifdate|N;t_savestep|N";
                listSchema.push_back(minimum);
        } else if (iRealTable=="parameters") {
                SKGModelTemplate def;
                def.id="default";
                def.name=i18nc("Noun, the default value of an item","Default");
                def.icon="edit-undo";
                def.schema="t_name;t_value";
                listSchema.push_back(def);
        } else if (iRealTable=="node") {
                SKGModelTemplate def;
                def.id="default";
                def.name=i18nc("Noun, the default value of an item","Default");
                def.icon="edit-undo";
                def.schema="t_name";
                listSchema.push_back(def);
        }

        return listSchema;
}

QString SKGDocument::getDisplay(const QString& iString) const
{
        QString output=iString.toLower();

        if (output.endsWith(QLatin1String("t_name"))) output=i18nc("Noun, the name of an item","Name");
        else if (output.endsWith(QLatin1String("d_date"))) output=i18nc("Noun, the date of an item","Date");
        else if (output.endsWith(QLatin1String("t_savestep"))) output=i18nc("Verb, save a document","Save");
        else if (output.endsWith(QLatin1String("t_value"))) output=i18nc("Noun, the value of an item","Value");
        else if (output.endsWith(QLatin1String("d_lastmodifdate"))) output=i18n("Last modification");
        else output=iString;
        return output;
}

QIcon SKGDocument::getIcon(const QString& iString) const
{
        Q_UNUSED(iString);
        return QIcon();
}

SKGServices::AttributeType SKGDocument::getAttributeType(const QString& iAttributeName) const
{
        SKGServices::AttributeType output=SKGServices::TEXT;
        if (iAttributeName.startsWith(QLatin1String("d_"))) output= SKGServices::DATE;
        else if (iAttributeName.startsWith(QLatin1String("i_"))) output= SKGServices::INTEGER;
        else if (iAttributeName.startsWith(QLatin1String("rd_")) || iAttributeName.startsWith(QLatin1String("rc_")) || iAttributeName.startsWith(QLatin1String("r_"))) output= SKGServices::LINK;
        else if (iAttributeName.startsWith(QLatin1String("f_"))) output= SKGServices::FLOAT;
        else if (iAttributeName.startsWith(QLatin1String("b_"))) output= SKGServices::BLOB;
        else if (iAttributeName=="id") output= SKGServices::ID;

        return output;
}

void SKGDocument::addValueInCache(const QString& iKey, const QString& iValue)
{
        cache[iKey]=iValue;
}

QString SKGDocument::getCachedValue(const QString& iKey) const
{
        return cache[iKey];
}

void SKGDocument::setBackupParameters(const QString& iPrefix, const QString& iSuffix)
{
        backupPrefix=iPrefix;
        backupSuffix=iSuffix;
}

QString SKGDocument::getTemporaryFile(const QString iFileName)
{
        QFileInfo fi(iFileName);
        return fi.absolutePath()+"/."+fi.fileName()+".wrk";
}

QString SKGDocument::getBackupFile(const QString iFileName)
{
        QString output;
        if (!backupPrefix.isEmpty() || !backupSuffix.isEmpty()) {
                QFileInfo fi(iFileName);
                output=fi.absolutePath()+'/'+backupPrefix+fi.fileName()+backupSuffix;
                output=output.replace("<DATE>", SKGServices::timeToString(QDateTime::currentDateTime()));
        }

        return output;
}

#include "skgdocument.moc"

