/***************************************************************************
 *   Copyright (C) 2008 by S. MANKOWSKI / G. DE BURE skrooge@mankowski.fr  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>  *
 ***************************************************************************/
/** @file
* This file implements classes SKGServices.
*
* @author Stephane MANKOWSKI / Guillaume DE BURE
 */
#include "skgservices.h"
#include "skgtraces.h"
#include "skgdocument.h"

#include <klocale.h>
#include <kglobal.h>

#include <sys/time.h>

#include <QSqlDatabase>
#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlError>
#include <QSqlDriver>
#include <QVariant>
#include <QRegExp>
#include <QFile>
#include <QtCrypto>

#include <sqlite3.h>

int SKGServices::SKGSqlTraces = SKGServices::stringToInt(SKGServices::getenv("SKGTRACESQL"));

SKGError SKGServices::lastCallbackError;

SKGServices::SKGServices():QObject() {}

SKGServices::~SKGServices() {}

QString SKGServices::getenv(const QString& iAttribute)
{
        return QString::fromUtf8(qgetenv(iAttribute.toUtf8().constData()));
}

QString SKGServices::intToString(int iNumber)
{
        QString output;
        output.setNum(iNumber);
        return output;
}

int SKGServices::stringToInt(const QString& iNumber)
{
        if (iNumber.isEmpty()) return 0;

        bool ok;
        int output=iNumber.toInt(&ok);
        if (!ok) {
                SKGTRACE << "WARNING: SKGServices::stringToInt(" << iNumber << ") failed" << endl;
        }

        return output;
}

QString SKGServices::stringToSqlString(const QString& iString)
{
        QString output=iString;
        output.replace('\'',"''");
        return output;
}

QString SKGServices::stringToHtml(const QString& iString)
{
        QString output=iString;
        output.replace('&',"&amp;"); //Must be done first
        output.replace('<',"&lt;");
        output.replace('>',"&gt;");
        output.replace('"',"&quot;");

        return output;
}

QString SKGServices::htmlToString(const QString& iString)
{
        QString output=iString;
        output.replace("&lt;", "<");
        output.replace("&gt;", ">");
        output.replace("&quot;", "\"");
        output.replace("&amp;", "&");

        return output;
}

QString SKGServices::stringsToCsv(const QStringList& iList, const QChar& iSeparator)
{
        QString output;
        int nb=iList.count();
        for (int i=0; i<nb; ++i) {
                output.append(SKGServices::stringToCsv(iList[i]));
                if (i<nb-1) output.append(iSeparator);
        }

        return output;
}

QString SKGServices::stringToCsv(const QString& iNumber)
{
        QString output=iNumber;
        output.replace('"',"#SKGDOUBLECOTE#");
        output.replace("#SKGDOUBLECOTE#", "\"\"");
        output='"'+output+'"';
        return output;
}

double SKGServices::stringToDouble(const QString& iNumber)
{
        if (iNumber.isEmpty()) return 0;

        bool ok;
        double output=iNumber.toDouble(&ok);
        if (!ok) {
                QString tmp=iNumber;
                tmp.replace(',', '.');
                output=tmp.toDouble(&ok);
                if (!ok) {
                        QString tmp=iNumber;
                        tmp.replace('.', ',');
                        output=tmp.toDouble(&ok);
                        if (!ok) {
                                QString tmp=iNumber;
                                tmp.remove(',');
                                output=tmp.toDouble(&ok);
                        }
                }
        }
        if (!ok) {
                SKGTRACE << "WARNING: SKGServices::stringToDouble(" << iNumber << ") failed" << endl;
        }
        return output;
}

QString SKGServices::doubleToString(double iNumber)
{
        QString output;
        output.setNum(iNumber,'g', 10);
        return output;
}

QString SKGServices::timeToString(const QDateTime& iDateTime)
{
        return iDateTime.toString("yyyy-MM-dd HH:mm:ss");
}

QString SKGServices::dateToSqlString(const QDateTime& iDateTime)
{
        return iDateTime.toString("yyyy-MM-dd");
}

QDateTime SKGServices::stringToTime(const QString& iDateString)
{
        QDateTime output=QDateTime::fromString (iDateString, "yyyy-MM-dd HH:mm:ss");
        if (!output.isValid()) {
                output=QDateTime::fromString (iDateString, "yyyy-MM-dd");
        }

        return output;
}

QStringList SKGServices::splitCSVLine(const QString& iString, const QChar& iSeparator, bool iCoteDefineBlock, QChar* oRealSeparator)
{
        QStringList items;
        QString item;
        bool isInBlock = false;
        QChar realSeparator=iSeparator;

        int nb=iString.length();
        for (int pos = 0; pos < nb; ++pos) {
                QChar c = iString.at(pos);
                if ( isInBlock ) {
                        if ( c == '\"' ) {
                                if (pos<nb-1 && iString.at(pos+1)=='\"') ++pos;
                                else {
                                        items.push_back(item);
                                        item="";
                                        isInBlock = false;

                                        ++pos; //To ignore next separator
                                        if (pos<nb) realSeparator=iString.at(pos); //To get the real separator
                                }
                        }

                        if (isInBlock) item+=c;
                } else  if (c=='\"' && iCoteDefineBlock) {
                        isInBlock = true;
                } else  if (QString(c)==realSeparator) {
                        items.push_back(item);
                        item="";
                        isInBlock = false;
                } else {
                        item+=c;
                }
        }

        if (!item.isEmpty() || (nb>0 && iString.at(nb-1)==realSeparator)) items.push_back(item);

        if (oRealSeparator) *oRealSeparator=realSeparator;

        return items;
}

QString SKGServices::getDateFormat(const QStringList& iDates)
{
        SKGTRACEIN(2, "SKGServices::getDateFormat");
        bool f_YYYY_MM_DD=true;
        bool f_YYYYMMDD=true;
        bool f_MM_DD_YY=true;
        bool f_DD_MM_YY=true;
        bool f_MM_DD_YYYY=true;
        bool f_DD_MM_YYYY=true;

        //Build regexp
        QRegExp rx("(.+)-(.+)-(.+)");

        //Check all dates
        int nb=iDates.count();
        for (int i=0; i<nb; ++i) {

                QString val=iDates.at(i);
                val=val.replace('/','-');
                val=val.replace("'20","-20");
                val=val.replace("' ","-200");
                val=val.replace('\'',"-20");
                val=val.replace('.','-');
                if (rx.indexIn(val)==-1) {
                        f_YYYY_MM_DD=true;
                        f_MM_DD_YY=true;
                        f_DD_MM_YY=true;
                        f_MM_DD_YYYY=true;
                        f_DD_MM_YYYY=true;

                        if (f_YYYYMMDD && val.length()!=8) f_YYYYMMDD=false;
                } else {
                        f_YYYYMMDD=false;

                        QString v1=rx.cap(1);
                        QString v2=rx.cap(2);
                        QString v3=rx.cap(3);

                        if (SKGServices::stringToInt(v1)>12) {
                                f_MM_DD_YY=false;
                                f_MM_DD_YYYY=false;
                        }

                        if (SKGServices::stringToInt(v2)>12) {
                                f_DD_MM_YY=false;
                                f_DD_MM_YYYY=false;
                        }

                        if (SKGServices::stringToInt(v1)>31 || SKGServices::stringToInt(v2)>31) {
                                f_MM_DD_YY=false;
                                f_MM_DD_YYYY=false;
                                f_DD_MM_YY=false;
                                f_DD_MM_YYYY=false;
                        }

                        if (SKGServices::stringToInt(v3)>31) {
                                f_YYYY_MM_DD=false;
                        }

                        if (v1.length()==4) {
                                f_MM_DD_YY=false;
                                f_DD_MM_YY=false;
                                f_MM_DD_YYYY=false;
                                f_DD_MM_YYYY=false;
                        } else {
                                f_YYYY_MM_DD=false;
                        }

                        if (v3.length()==4) {
                                f_YYYY_MM_DD=false;
                                f_MM_DD_YY=false;
                                f_DD_MM_YY=false;
                        } else {
                                f_MM_DD_YYYY=false;
                                f_DD_MM_YYYY=false;
                        }
                }
        }

        if (f_YYYYMMDD) return "YYYYMMDD";
        else if (f_YYYY_MM_DD) return "YYYY-MM-DD";
        else if (f_DD_MM_YY && f_MM_DD_YY)
	{
	    KLocale* locale=KGlobal::locale();
	    if(locale)
	    {
	      QString sFormat=locale->dateFormatShort ();
	      if(sFormat.startsWith(QLatin1String("%m")) || sFormat.startsWith(QLatin1String("%n"))) return "MM-DD-YY";
	    }
	    return "DD-MM-YY";
	}
        else if (f_DD_MM_YY) return "DD-MM-YY";
        else if (f_MM_DD_YY) return "MM-DD-YY";
        else if (f_DD_MM_YYYY && f_MM_DD_YYYY)
	{
	    KLocale* locale=KGlobal::locale();
	    if(locale)
	    {
	      QString sFormat=locale->dateFormatShort ();
	      if(sFormat.startsWith(QLatin1String("%m")) || sFormat.startsWith(QLatin1String("%n"))) return "MM-DD-YYYY";
	    }
	    return "DD-MM-YYYY";
	}
        else if (f_DD_MM_YYYY) return "DD-MM-YYYY";
        else if (f_MM_DD_YYYY) return "MM-DD-YYYY";

        return "";
}

QString SKGServices::dateToSqlString(const QString& iDate, const QString& iFormat)
{
        QString input=iDate;

        QString YYYY="0000";
        QString MM="00";
        QString DD="00";
        if (iFormat=="YYYYMMDD") {
                YYYY=input.mid(0,4);
                MM=input.mid(4,2);
                DD=input.mid(6,2);

        } else {
                QString val=iDate;
                val=val.replace('/','-');
                val=val.replace("'20","-20");
                val=val.replace("' ","-200");
                val=val.replace('\'',"-20");
                val=val.replace('.','-');
                QRegExp rx("(.+)-(.+)-(.+)");
                if (rx.indexIn(val)!=-1) {
                        QString v1=rx.cap(1);
                        QString v2=rx.cap(2);
                        QString v3=rx.cap(3);
                        if (iFormat=="YYYY-MM-DD") {
                                YYYY=v1;
                                MM=v2;
                                DD=v3;
                        } else if (iFormat=="MM/DD/YY" || iFormat=="MM-DD-YY" || iFormat=="MM/DD/YYYY" || iFormat=="MM-DD-YYYY") {
                                MM=v1;
                                DD=v2;
                                YYYY=v3;
                        } else if (iFormat=="DD/MM/YY" || iFormat=="DD-MM-YY" || iFormat=="DD/MM/YYYY" || iFormat=="DD-MM-YYYY") {
                                DD=v1;
                                MM=v2;
                                YYYY=v3;
                        }
                }
        }

        if (MM.length()==1) MM='0'+MM;
        if (DD.length()==1) DD='0'+DD;
        if (YYYY.length()==1) YYYY='0'+YYYY;
        if (YYYY.length()==2) {
                if (stringToInt(YYYY)>70) YYYY="19"+YYYY;
                else YYYY="20"+YYYY;
        }

        QString date=YYYY+'-'+MM+'-'+DD;
        date.replace(' ','0');
        return dateToSqlString(stringToTime(date));
}

QStringList SKGServices::tableToDump(const SKGStringListList& iTable, DumpMode iMode)
{
        SKGTRACEIN(10, "SKGServices::tableToDump");
        //initialisation
        QStringList oResult;

        //Compute max size of each column
        int *maxSizes=NULL;
        if (iMode==DUMP_TEXT) {
                int nb=iTable.count();
                for (int i=0; i<nb; ++i) {
                        QStringList line=iTable.at(i);

                        if (maxSizes==NULL) {
                                int size=line.size();
                                maxSizes=new int[size];
                                if (maxSizes) {
                                        for (int j=0; j<size; ++j) {
                                                maxSizes[j]=0;
                                        }
                                }
                        }

                        int nb2=line.size();
                        for (int j = 0; j < nb2; ++j) {
                                QString s=line.at(j);
                                if (s.length()>maxSizes[j]) maxSizes[j]=s.length();
                        }
                }
        }

        //dump
        int nb=iTable.count();
        for (int i=0; i<nb; ++i) {
                QString lineFormated;
                if (iMode==DUMP_TEXT) {
                        lineFormated="| ";
                }

                QStringList line=iTable.at(i);
                int nb2=line.size();
                for (int j = 0; j < nb2; ++j) {
                        QString s=line.at(j);
                        s.remove('\n');

                        if (iMode==DUMP_CSV) {
                                if (j>0) lineFormated+=';';
                                lineFormated+=stringToCsv(s);
                        } else {
                                s=s.leftJustified(maxSizes[j], ' ');
                                lineFormated+=s+" | ";
                        }
                }
                oResult.push_back(lineFormated);
        }

        //delete
        if (maxSizes) {
                delete [] maxSizes;
                maxSizes=NULL;
        }

        return oResult;
}

QString SKGServices::getRealTable(const QString& iTable)
{
        QString output=iTable;
        if (output.length() > 2 && output.startsWith(QLatin1String("v_"))) {
                output = output.mid(2, output.length() - 2);

                int pos=output.indexOf("_");
                if (pos!=-1) output=output.left(pos);
        }

        return output;
}

SKGError SKGServices::cryptFile(const QString& iFileSource, const QString& iFileTarget, const QString& iPassword, bool iEncrypt)
{
        SKGError err;
        SKGTRACEINRC(10, "SKGServices::cryptFile", err);
        SKGTRACEL(10) << "Input parameter [iFileSource]=[" << iFileSource << ']' << endl;
        SKGTRACEL(10) << "Input parameter [iFileTarget]=[" << iFileTarget << ']' << endl;
        if (iPassword.isEmpty()) {
                if (!QFile::copy(iFileSource, iFileTarget)) {
                        err = SKGError(ERR_FAIL, i18n("copy of %1 to %2 failed",iFileSource,iFileTarget));
                }
        } else {
                QCA::Initializer init;

                // Create a random key - you'd probably use one from another
                // source in a real application
                QCA::SymmetricKey key(QByteArray("skrooge"));

                // Create a random initialisation vector - you need this
                // value to decrypt the resulting cipher text, but it
                // need not be kept secret (unlike the key).
                QCA::InitializationVector iv(iPassword.toAscii());

                // create a 128 bit AES cipher object using Cipher Block Chaining (CBC) mode
                QCA::Cipher cipher(QString("aes128"),QCA::Cipher::CBC,
                                   // use Default padding, which is equivalent to PKCS7 for CBC
                                   QCA::Cipher::DefaultPadding,
                                   iEncrypt ? QCA::Encode : QCA::Decode,
                                   key, iv);
                QFile fileInput(iFileSource);
                if (!fileInput.open(QIODevice::ReadOnly)) err=SKGError(ERR_INVALIDARG, i18n("Open file [%1] failed",iFileSource));
                else {
                        QByteArray input = fileInput.readAll();

                        // we use the cipher object to encrypt the argument we passed in
                        // the result of that is returned - note that if there is less than
                        // 16 bytes (1 block), then nothing will be returned - it is buffered
                        // update() can be called as many times as required.
                        QCA::SecureArray u = cipher.update(input);

                        // We need to check if that update() call worked.
                        if (!cipher.ok()) err=SKGError(ERR_FAIL, i18n("Encryption failed"));

                        // output the results of that stage
                        if (err.isSucceeded()) {
                                QFile fileOutput(iFileTarget);
                                if (!fileOutput.open(QIODevice::WriteOnly)) err=SKGError(ERR_INVALIDARG, i18n("Write file [%1] failed",iFileTarget));
                                else {
                                        fileOutput.write(u.toByteArray());

                                        // Because we are using PKCS7 padding, we need to output the final (padded) block
                                        // Note that we should always call final() even with no padding, to clean up
                                        QCA::SecureArray f = cipher.final();

                                        // Check if the final() call worked
                                        if (!cipher.ok()) err=SKGError(ERR_FAIL, i18n("Encryption failed"));

                                        // and output the resulting block. The ciphertext is the results of update()
                                        // and the result of final()
                                        fileOutput.write(f.toByteArray());

                                        //Close the file
                                        fileOutput.flush();
                                        fileOutput.close();
                                }
                        }
                }
        }
        return err;
}

SKGError SKGServices::copySqliteDatabase(QSqlDatabase* iFileDb, QSqlDatabase* iMemoryDb, bool iFromFileToMemory)
{
        SKGError err;
        SKGTRACEINRC(10, "SKGServices::copySqliteDatabase", err);
        if (iFileDb && iMemoryDb) {
                SKGTRACEL(20) << "Input parameter [iFileDb]=[" << iFileDb->databaseName() << ']' << endl;
                SKGTRACEL(20) << "Input parameter [iMemoryDb]=[" << iMemoryDb->databaseName() << ']' << endl;
                SKGTRACEL(10) << "Input parameter [iFromFileToMemory]=[" << (iFromFileToMemory ? "FILE->MEMORY" : "MEMORY->FILE") << ']' << endl;

                QString dbFileName=iFileDb->databaseName();

                // Copy the schema
                SKGTRACEL(20) << "Coping the schema..." << endl;
                SKGStringListList listSqlOrder;
                err=SKGServices::executeSelectSqliteOrder((iFromFileToMemory ? iFileDb: iMemoryDb),
                                "SELECT sql FROM sqlite_master WHERE sql NOT NULL and name NOT IN ('sqlite_stat1', 'sqlite_sequence')",
                                listSqlOrder);

                int nb=listSqlOrder.count();
                for (int i=1; err.isSucceeded() && i<nb; ++i) { //Forget header
                        QString val=listSqlOrder.at(i).at(0);
                        err=SKGServices::executeSqliteOrder((iFromFileToMemory ? iMemoryDb: iFileDb), val);
                }

                // Attach the file dbFileName to the target
                if (err.isSucceeded()) {
                        SKGTRACEL(20) << "Attaching the file..." << endl;
                        err = SKGServices::executeSqliteOrder(iMemoryDb, "ATTACH DATABASE '" + dbFileName + "' as source");
                        if (err.isSucceeded()) {
                                // Copy the DATA from the source to the target
                                SKGTRACEL(20) << "Coping data..." << endl;
                                err = SKGServices::executeSqliteOrder(iMemoryDb, "BEGIN");
                                if (err.isSucceeded()) {
                                        SKGStringListList listSqlOrder;
                                        err=SKGServices::executeSelectSqliteOrder(iMemoryDb,
                                                        "SELECT name FROM source.sqlite_master WHERE type='table' and name NOT IN ('sqlite_stat1', 'sqlite_sequence')",
                                                        listSqlOrder);
                                        int nb=listSqlOrder.count();
                                        for (int i=1; err.isSucceeded() && i<nb; ++i) { //Forget header
                                                QString val=listSqlOrder.at(i).at(0);
                                                if (iFromFileToMemory)  err=SKGServices::executeSqliteOrder(iMemoryDb, "insert into main."+val+" select * from source."+val);
                                                else  err=SKGServices::executeSqliteOrder(iMemoryDb, "insert into source."+val+" select * from main."+val);
                                        }

                                }
                                if (err.isSucceeded())  err = SKGServices::executeSqliteOrder(iMemoryDb, "COMMIT");

                                //Detach
                                SKGError err2=SKGServices::executeSqliteOrder(iMemoryDb, "DETACH DATABASE source");
                                if (err.isSucceeded() && err2.isFailed()) err=err2;

                                //Check if created file exists
                                if (err.isSucceeded() && !iFromFileToMemory && !QFile(dbFileName).exists()) {
                                        //Set error message
                                        err.setReturnCode(ERR_FAIL);
                                        err.setMessage(i18n("Creation file [%1] failed",dbFileName));
                                }
                        }
                }
                if (err.isFailed()) err.addError(SQLLITEERROR + ERR_FAIL, i18n("%1 failed", QString("copySqliteDatabase::copySqliteDatabase()")));
        }
        return err;
}

SKGError SKGServices::executeSqliteOrder(const SKGDocument* iDocument, const QString& iSqlOrder, int* iLastId)
{
        SKGError err;
        _SKGTRACEINRC(10, "SKGServices::ExecuteSqlOrder(SKGDocument)", err);
        if (iDocument == NULL) {
                err = SKGError(ERR_POINTER, i18n("No database defined"));
        } else {
                err = SKGServices::executeSqliteOrder(iDocument->getDatabase(), iSqlOrder, iLastId);
        }
        return err;
}

SKGError SKGServices::executeSqliteOrder(QSqlDatabase* iDb, const QString& iSqlOrder, int* iLastId)
{
        SKGError err;
        _SKGTRACEINRC(10, "SKGServices::executeSqliteOrder(QSqlDatabase)", err);
        SKGTRACEL(20) << "Input parameter [iSqlOrder]=[" << iSqlOrder << ']' << endl;

        if (iDb == NULL) {
                err = SKGError(ERR_POINTER, i18n("No database defined"));
        } else {
                QStringList listSqlOrder=iSqlOrder.split(";;", QString::SkipEmptyParts);
                int nb=listSqlOrder.size();
                if (nb>1) {
                        for (int i=0; err.isSucceeded() && i<nb ; ++i) {
                                QString sql=listSqlOrder.at(i).trimmed();
                                if (!sql.isEmpty()) {
                                        err=SKGServices::executeSqliteOrder(iDb, sql, iLastId);
                                }
                        }
                } else {
                        QSqlQuery query(QString(), *iDb);
                        double elapse=0;
                        if (SKGServices::SKGSqlTraces)  elapse=SKGServices::getMicroTime();

                        if (!query.exec(iSqlOrder)) {
                                QSqlError sqlError=query.lastError();
                                if (sqlError.number()!=SQLITE_CONSTRAINT) {
                                        SKGTRACE << "WARNING: " << iSqlOrder << endl;
                                        SKGTRACE << "         returns :" << sqlError.text() << endl;
                                }

                                QString msg=iSqlOrder+':'+sqlError.text();
                                err = SKGError(SQLLITEERROR + sqlError.number(), msg);

                                QVariant v = iDb->driver()->handle();
                                if (v.isValid() && qstrcmp(v.typeName(), "sqlite3*")==0) {
                                        sqlite3* handle = *static_cast<sqlite3 **>(v.data());
                                        if (handle != NULL) { // check that it is not NULL
                                                QString dmsg=sqlite3_errmsg(handle);
                                                if (!dmsg.isEmpty()) err.addError(SQLLITEERROR + sqlError.number(), dmsg);
                                        }
                                }

                        } else {
                                if (iLastId) *iLastId=query.lastInsertId().toInt();
                        }
                        if (SKGServices::SKGSqlTraces) {
                                elapse=SKGServices::getMicroTime() - elapse;
                                if (elapse>=SKGServices::SKGSqlTraces) {
                                        SKGTRACE << "executeSqliteOrder :" << iSqlOrder << " TIME=" << elapse << " ms" << endl;
                                }
                        }
                }
        }
        return err;
}

SKGError SKGServices::dumpSelectSqliteOrder(QSqlDatabase* iDb, const QString& iSqlOrder, QTextStream* oStream, DumpMode iMode)
{
        SKGError err;
        _SKGTRACEINRC(10, "SKGServices::dumpSelectSqliteOrder(QSqlDatabase)", err);
        SKGTRACEL(20) << "Input parameter [iSqlOrder]=[" << iSqlOrder << ']' << endl;

        //initialisation
        QStringList oResult;
        err=SKGServices::dumpSelectSqliteOrder(iDb, iSqlOrder, oResult, iMode);
        if (err.isSucceeded()) {
                //dump
                int nb=oResult.size();
                for (int i = 0; i < nb; ++i) {
                        if (oStream==NULL)  SKGTRACESUITE << oResult.at(i) << endl;
                        else  *oStream << oResult.at(i) << endl;
                }
        }
        return err;
}

SKGError SKGServices::dumpSelectSqliteOrder(const SKGDocument* iDocument, const QString& iSqlOrder, QTextStream* oStream, DumpMode iMode)
{
        SKGError err;
        _SKGTRACEINRC(10, "SKGServices::dumpSelectSqliteOrder(SKGDocument)", err);

        if (iDocument == NULL) err = SKGError(ERR_POINTER, i18n("No database defined"));
        else  err = SKGServices::dumpSelectSqliteOrder(iDocument->getDatabase(), iSqlOrder, oStream, iMode);
        return err;
}

SKGError SKGServices::dumpSelectSqliteOrder(QSqlDatabase* iDb, const QString& iSqlOrder, QString& oResult, DumpMode iMode)
{
        SKGError err;
        _SKGTRACEINRC(10, "SKGServices::dumpSelectSqliteOrder(QSqlDatabase)", err);
        //initialisation
        oResult="";

        QStringList oResultTmp;
        err=SKGServices::dumpSelectSqliteOrder(iDb, iSqlOrder, oResultTmp, iMode);
        if (err.isSucceeded()) {
                //dump
                int nb=oResultTmp.size();
                for (int i = 0; i < nb; ++i) {
                        oResult+=oResultTmp.at(i)+'\n';
                }
        }
        return err;
}

SKGError SKGServices::dumpSelectSqliteOrder(const SKGDocument* iDocument, const QString& iSqlOrder, QString& oResult, DumpMode iMode)
{
        SKGError err;
        _SKGTRACEINRC(10, "SKGServices::dumpSelectSqliteOrder(SKGDocument)", err);
        if (iDocument == NULL) err = SKGError(ERR_POINTER, i18n("No database defined"));
        else  err = SKGServices::dumpSelectSqliteOrder(iDocument->getDatabase(), iSqlOrder, oResult, iMode);
        return err;
}

SKGError SKGServices::dumpSelectSqliteOrder(QSqlDatabase* iDb, const QString& iSqlOrder, QStringList& oResult, DumpMode iMode)
{
        SKGError err;
        _SKGTRACEINRC(10, "SKGServices::dumpSelectSqliteOrder(QSqlDatabase)", err);

        //Execution of sql order
        SKGStringListList oResultTmp;
        err=executeSelectSqliteOrder(iDb, iSqlOrder, oResultTmp);
        if (err.isSucceeded()) oResult=tableToDump(oResultTmp, iMode);
        return err;
}

SKGError SKGServices::dumpSelectSqliteOrder(const SKGDocument* iDocument, const QString& iSqlOrder, QStringList& oResult, DumpMode iMode)
{
        SKGError err;
        _SKGTRACEINRC(10, "SKGServices::dumpSelectSqliteOrder(SKGDocument)", err);
        if (iDocument == NULL) err = SKGError(ERR_POINTER, i18n("No database defined"));
        else err = SKGServices::dumpSelectSqliteOrder(iDocument->getDatabase(), iSqlOrder, oResult, iMode);
        return err;
}

SKGError SKGServices::executeSelectSqliteOrder(QSqlDatabase* iDb, const QString& iSqlOrder, SKGStringListList& oResult)
{
        SKGError err;
        _SKGTRACEINRC(10, "SKGServices::executeSelectSqliteOrder(QSqlDatabase)", err);
        //initialisation
        oResult.clear();

        if (iDb == NULL) err = SKGError(ERR_POINTER, i18n("No database defined"));
        else {
                QSqlQuery query(QString(), *iDb);
                double elapse=0;
                if (SKGServices::SKGSqlTraces)  elapse=SKGServices::getMicroTime();

                if (!query.exec(iSqlOrder)) {
                        QSqlError sqlError=query.lastError();
                        SKGTRACE << "WARNING: " << iSqlOrder << endl;
                        SKGTRACE << "         returns :" << sqlError.text() << endl;
                        err = SKGError(SQLLITEERROR + sqlError.number(), iSqlOrder+':'+sqlError.text());
                } else {
                        if (SKGServices::SKGSqlTraces) {
                                double elapse1=SKGServices::getMicroTime() - elapse;
                                if (elapse1>=SKGServices::SKGSqlTraces) {
                                        SKGTRACE << "executeSqliteOrder :" << iSqlOrder << " TIME=" << elapse1 << " ms" << endl;
                                }
                        }

                        //Addition of column names
                        QSqlRecord rec = query.record();
                        QStringList line;
                        int index=0;
                        while (index!=-1) {
                                QString val=rec.fieldName(index);
                                if (!val.isEmpty()) {
                                        line.push_back(val);
                                        ++index;
                                } else {
                                        index=-1;
                                }
                        }
                        oResult.push_back(line);

                        //Addition of rows
                        while (query.next()) {
                                QStringList line;
                                int index=0;
                                while (index!=-1) {
                                        QVariant val=query.value(index);
                                        if (val.isValid()) {
                                                line.push_back(val.toString());
                                                ++index;
                                        } else {
                                                index=-1;
                                        }
                                }
                                oResult.push_back(line);
                        }
                        if (SKGServices::SKGSqlTraces) {
                                double elapse1=SKGServices::getMicroTime() - elapse;
                                if (elapse1>=SKGServices::SKGSqlTraces) {
                                        SKGTRACE << "executeSqliteOrder (with fetch) :" << iSqlOrder << " TIME=" << elapse1 << " ms" << endl;
                                }
                        }
                }
        }
        return err;
}

SKGError SKGServices::executeSelectSqliteOrder(const SKGDocument* iDocument, const QString& iSqlOrder, SKGStringListList& oResult)
{
        SKGError err;
        _SKGTRACEINRC(10, "SKGServices::executeSelectSqliteOrder(SKGDocument)", err);
        oResult.clear();
        if (iDocument == NULL) err = SKGError(ERR_POINTER, i18n("No database defined"));
        else  err = SKGServices::executeSelectSqliteOrder(iDocument->getDatabase(), iSqlOrder, oResult);
        return err;
}

SKGError SKGServices::getDistinctValues(const SKGDocument* iDocument, const QString& iTable, const QString& iAttribute, const QString& iWhereClause, QStringList& oResult)
{
        SKGError err;
        _SKGTRACEINRC(10, "SKGServices::getDistinctValues", err);
        //initialisation
        oResult.clear();

        //Search
        SKGStringListList temporaryResult;
        err = SKGServices::executeSelectSqliteOrder((SKGDocument*) iDocument,
                        "SELECT DISTINCT " + iAttribute +
                        " FROM " + iTable + " WHERE (" +
                        (iWhereClause.length() ? iWhereClause : "1=1") +
                        ") ORDER BY " + iAttribute
                        //Correction bug 202167 vvv
                        +" COLLATE NOCASE"
                        //Correction bug 202167 ^^^
                        , temporaryResult);
        if (err.isSucceeded()) {
                SKGStringListListIterator it = temporaryResult.begin();
                ++it; //to forget column name
                for (; it != temporaryResult.end(); ++it) {
                        oResult.push_back(*(it->begin()));
                }
        }

        return err;
}

SKGError SKGServices::getDistinctValues(const SKGDocument* iDocument, const QString& iTable, const QString& iAttribute, QStringList& oResult)
{
        return getDistinctValues(iDocument, iTable, iAttribute,
                                 iAttribute + " IS NOT NULL AND " + iAttribute + "!=''",
                                 oResult);
}

SKGError SKGServices::getTablesList(const SKGDocument* iDocument, QStringList& oResult)
{
        return getDistinctValues(iDocument, "sqlite_master", "name",
                                 "type='table' AND name NOT LIKE 'sqlite_%'",
                                 oResult);;
}

SKGError SKGServices::getAttributesDescription(const SKGDocument* iDocument, const QString& iTable, SKGAttributesList& oResult)
{
        SKGError err;
        _SKGTRACEINRC(10, "SKGServices::getAttributesDescription", err);
        //initialisation
        oResult.clear();

        //Search
        SKGStringListList temporaryResult;
        err = SKGServices::executeSelectSqliteOrder((SKGDocument*) iDocument, "PRAGMA table_info( " + iTable + " );",temporaryResult);
        if (err.isSucceeded() && iDocument) {
                int nblines=temporaryResult.count();
                QString realTable=SKGServices::getRealTable(iTable);

                for (int i=1; i<nblines; ++i) { // the first one is ignored because it is the headers
                        QStringList line=temporaryResult[i];

                        SKGAttributeInfo attribute;
                        attribute.name=line[1];

                        QString attname=realTable+'.'+attribute.name;
                        attribute.display=iDocument->getDisplay(attname);
                        if (attribute.display==attname) {
                                attribute.display="";
                        }
                        attribute.icon=iDocument->getIcon(attname);
                        attribute.type=iDocument->getAttributeType(attribute.name);
                        attribute.notnull=(line[3]=="0");
                        attribute.defaultvalue=line[4];
                        oResult.push_back(attribute);
                }
        }

        return err;
}

SKGError SKGServices::getAttributesList(const SKGDocument* iDocument, const QString& iTable, QStringList& oResult)
{
        SKGError err;
        _SKGTRACEINRC(10, "SKGServices::getAttributesList", err);
        oResult.clear();
        SKGAttributesList attDesc;
        err=SKGServices::getAttributesDescription(iDocument, iTable, attDesc);
        int nblines=attDesc.count();
        for (int i=0; err.isSucceeded() && i<nblines; ++i) {
                oResult.push_back(attDesc[i].name);
        }
        return err;
}

double SKGServices::getMicroTime()
{
#ifdef Q_OS_WIN
        return static_cast<double>(GetTickCount());
#else
        struct timeval tv;
        struct timezone tz;

        //get time
        gettimeofday(&tv, &tz);

        //return time
        return (static_cast<double>(1000.0 * tv.tv_sec)) + (static_cast<double>(tv.tv_usec / 1000));
#endif
}

#include "skgservices.moc"


